@@ -25,10 +25,10 @@ import numpy as np
 
 __all__ = ['get_session','set_session','get_session_value', 'get_trident_dir', 'get_signature', 'epsilon', 'floatx','Signature','import_or_install',
            'check_keys', 'if_else', 'camel2snake', 'snake2camel', 'to_onehot', 'to_list', 'addindent', 'format_time',
-           'get_time_suffix', 'get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime',
+           'get_time_suffix', 'get_file_modified_time','get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime',
            'next_prime', 'prev_prime', 'nearest_prime', 'PrintException', 'unpack_singleton', 'enforce_singleton',
            'OrderedDict', 'get_python_function_arguments', 'map_function_arguments', 'ClassfierType', 'PaddingMode',
-           'DataRole',
+           'DataRole','is_numpy',
 
            'ExpectDataType', 'GetImageMode', 'split_path', 'make_dir_if_need', 'sanitize_path', 'ShortcutMode',
            'DataSpec', 'get_args_spec','update_signature', 'get_gpu_memory_map']
@@ -352,8 +352,9 @@ def PrintException():
     linecache.checkcache(filename)
     line = linecache.getline(filename, lineno, f.f_globals)
     print('EXCEPTION IN ({}, LINE {} "{}"): {}\n'.format(filename, lineno, line.strip(), exc_obj))
-    traceback.print_tb(tb, limit=1, file=sys.stdout)
-    traceback.print_exception(exc_type, exc_obj, tb, limit=2, file=sys.stdout)
+    traceback.print_exc(limit=None, file=sys.stderr)
+    # traceback.print_tb(tb, limit=1, file=sys.stdout)
+    # traceback.print_exception(exc_type, exc_obj, tb, limit=2, file=sys.stdout)
 
 
 class OrderedDict(collections.OrderedDict):
@@ -455,6 +456,26 @@ def get_signature(fn,name=None):
 
     return signature
 
+class Initializer(object):
+  """Initializer base class: all initializers inherit from this class.
+  """
+  def __call__(self, shape, dtype=None):
+    """Returns a tensor object initialized as specified by the initializer.
+    Args:
+      shape: Shape of the tensor.
+      dtype: Optional dtype of the tensor. If not provided will return tensor
+       of `tf.float32`.
+
+    """
+    raise NotImplementedError
+
+  def get_config(self):
+    """Returns the configuration of the initializer as a JSON-serializable dict.
+    Returns:
+      A JSON-serializable Python dict.
+    """
+    return {}
+
 
 class AverageMeter(object):
     """Computes and stores the average and current value"""
@@ -575,7 +596,8 @@ def to_list(x):
         except:
             return x.tolist()
 
-
+def is_numpy(x):
+    return isinstance(x,np.ndarray)
 
 
 def unpack_singleton(x):
@@ -713,6 +735,28 @@ def get_time_suffix():
     prefix = str(datetime.datetime.fromtimestamp(time.time())).replace(' ', '').replace(':', '').replace('-', '').replace( '.', '')
     return prefix
 
+def get_file_modified_time(file_path):
+    """
+    Try to get the date that a file was modified, falling back to when it was
+    last modified if that isn't possible.
+    See http://stackoverflow.com/a/39501288/1709587 for explanation.
+    """
+    t=None
+    try:
+        t= os.path.getmtime(file_path)
+    except :
+        stat = os.stat(file_path)
+        try:
+            t= stat.st_birthtime
+        except AttributeError:
+            # We're probably on Linux. No easy way to get creation dates here,
+            # so we'll settle for when its content was last modified.
+            t= stat.st_mtime
+    if t is not None:
+        return datetime.datetime.fromtimestamp(t)
+    else:
+        return None
+
 
 def get_function(fn_name, module_paths=None):
     """
@@ -735,15 +779,21 @@ def get_function(fn_name, module_paths=None):
     """
     if callable(fn_name):
         return fn_name
-    fn = locate(fn_name)
-    if (fn is None) and (module_paths is not None):
+    fn=None
+    if (fn_name is not None) and (module_paths is not None):
         for module_path in module_paths:
             fn = locate('.'.join([module_path, fn_name]))
             if fn is not None:
                 break
+
     if fn is None:
-        raise ValueError("Method not found in {}: {}".format(module_paths, fn_name))
-    return fn  # type: ignore
+        fn = locate(fn_name)
+        if fn is not None:
+            return fn
+        else:
+            return None
+    else:
+        return fn  # type: ignore
 
 
 def get_class(class_name, module_paths=None):
@@ -765,13 +815,15 @@ def get_class(class_name, module_paths=None):
             :attr:`module_paths`.
 
     """
-    class_ = locate(class_name)
-    if (class_ is None) and (module_paths is not None):
+    class_ =None
+    if (class_name is not None) and (module_paths is not None):
         for module_path in module_paths:
             class_ = locate('.'.join([module_path, class_name]))
             if class_ is not None:
                 break
+
     if class_ is None:
+        class_ = locate(class_name)
         raise ValueError("Class not found in {}: {}".format(module_paths, class_name))
     return class_  # type: ignore
 
@@ -953,41 +1005,6 @@ def map_function_arguments(params, params_dict, *args, **kwargs):
 
 
 
-class OrderedDict(collections.OrderedDict):
-    """ more easy-to-use OrderedDict"""
-    def __init__(self, *args, **kwds):
-        super(OrderedDict, self).__init__(*args, **kwds)
-
-    @property
-    def key_list(self):
-        """
-        Returns:
-            list of keys
-
-        """
-        return list(super().keys())
-
-    @property
-    def value_list(self):
-        """
-        Returns:
-            list of values
-
-        """
-        return list(super().values())
-
-    @property
-    def item_list(self):
-        """
-        Returns:
-            list of items
-
-        """
-        return list(super().items())
-
-    def __repr__(self):
-        return '{ ' + (', '.join(['{0}: {1}'.format(k, v if v is not None else 'none') for k, v in self.item_list])) + ' }'
-
 
 class ClassfierType(Enum):
     dense = 'dense'
@@ -1036,6 +1053,7 @@ class ExpectDataType(Enum):
     landmarks = 'landmarks'
     random_noise = 'random_noise'
     classification_label = 'classification_label'
+    corpus='corpus'
 
 
 class GetImageMode(Enum):
