@@ -59,7 +59,9 @@ else:
             import tensorflow
             os.environ['TRIDENT_BACKEND'] = 'tensorflow'
         except:
-            pass
+            import_or_install('onnxruntime')
+
+            os.environ['TRIDENT_BACKEND'] = 'onnx'
 
 
 if _session.backend == 'pytorch':
@@ -83,9 +85,27 @@ elif _session.backend == 'tensorflow':
     _session.backend = 'tensorflow'
     _session.image_data_format = 'channels_last'
     _session.image_channel_order = 'rgb'
+    os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
+    import tensorflow as tf
+
+    gpus = tf.config.list_physical_devices('GPU')
+    if len(gpus)>0:
+        os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"  # see issue #152
+        os.environ["CUDA_VISIBLE_DEVICES"] = "0"
+        tf.config.set_visible_devices(gpus[0], 'GPU')
+        tf.config.experimental.set_memory_growth(gpus[0], True)
+        tf.device('/gpu:0')
 
 
 
+elif _session.backend == 'onnx':
+    stdout.write('Using ONNX backend.\n')
+    stdout.write('Image Data Format: channels_first.\n')
+    stdout.write('Image Channel Order: rgb.\n')
+    _session.backend = 'onnx'
+    _session.image_data_format = 'channels_first'
+    _session.image_channel_order = 'rgb'
+
 if 'TRIDENT_IMG_BACKEND' in os.environ:
     _image_backend = os.environ['TRIDENT_IMG_BACKEND']
     if _image_backend and _session.image_backend!=_image_backend:
