@@ -50,6 +50,8 @@ class ModelBase(object):
     def __init__(self, inputs=None,  input_shape=None,output=None, name='', **kwargs):
         if isinstance(inputs,tuple) and isinstance(inputs[0],int):
             input_shape.inputs=inputs,input_shape
+        self.batch_index = 0
+        self.filter_index = 1
         self.inputs = OrderedDict()
         self._outputs = OrderedDict()
         self._targets = OrderedDict()
@@ -107,7 +109,7 @@ class ModelBase(object):
                                  'stop_update': 0, 'retain_graph': False}
         self.training_context['losses']['total_losses'] = []
 
-        self._initial_graph(inputs, output, input_shape)
+        self._initial_graph(inputs, input_shape,output)
 
     @property
     def model(self):
@@ -118,13 +120,14 @@ class ModelBase(object):
 
         self._outputs = OrderedDict()
         self._targets = OrderedDict()
+        inp_shape = copy.deepcopy(self.inputs.value_list[0])
         if isinstance(value, Layer):
-            inp_shape = copy.deepcopy(self.inputs.value_list[0])
+
             self.inputs = OrderedDict()
-            self._initial_graph(output=value, input_shape=inp_shape)
+            self._initial_graph(input_shape=inp_shape,output=value)
         elif isinstance(value, np.ndarray) or 'tensor' in value.__name__.lower():
             self.inputs = OrderedDict()
-            self._initial_graph(output= to_tensor(value), input_shape= int_shape(value)[1:])
+            self._initial_graph(input_shape=inp_shape,output= to_tensor(value))
         else:
             raise ValueError('Only Layer, Module, Image and Tensor can be valid model')
 
@@ -506,16 +509,24 @@ class ModelBase(object):
 
 
         progress_bar(self.training_context['current_batch'], self.training_context['total_batch'],
-                 'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.3e}| epoch: {3}'.format(
+                 'Loss: {0:<8.5f}| {1} | learning rate: {2:<10.3e}| epoch: {3}'.format(
                      np.array(self.batch_loss_history['total_losses'][-1 * slice_cnt:]).mean(), ','.join(metric_strings), self.training_context['current_lr'],
                      self.training_context['current_epoch']), name=self.name)
 
     def print_epoch_progress(self, print_epoch_progress_frequency):
+        metric_strings=[]
+        for k, v in self.epoch_metric_history.items():
+            format_string='.3%'
+            metric_value=to_numpy(v)[-1]
+            if metric_value>5:
+                format_string = '.3f'
+            elif metric_value<1e-4:
+                format_string = '.3e'
+            metric_strings.append('{0}: {1:<8{2}}'.format(k,metric_value,format_string))
+
         progress_bar(self.training_context['current_epoch'], self.training_context['total_epoch'],
                      'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.3e}'.format(
-                         np.array(self.epoch_loss_history['total_losses']).mean(), ','.join(
-                             ['{0}: {1:<8.3%}'.format(k, np.array(v).mean()) for k, v in
-                              self.epoch_metric_history.items()]), self.training_context['current_lr']), name=self.name)
+                         np.array(self.epoch_loss_history['total_losses'])[-1], ','.join(metric_strings), self.training_context['current_lr']), name=self.name)
 
     def train_model(self, train_data, test_data, current_epoch, current_batch, total_epoch, total_batch,
                     is_collect_data=True, is_print_batch_progress=True, is_print_epoch_progress=True,
@@ -534,6 +545,8 @@ class ModelBase(object):
             self.training_context['test_data'] = test_data
 
 
+
+
             self.sample_collect_history.append(1 if is_collect_data else 0)
 
             if self.training_context['current_batch'] == 0:
@@ -548,7 +561,8 @@ class ModelBase(object):
                     self.training_context['grads_state'] = OrderedDict()
                     self.training_context['grads_state']['first_layer'] = []
                     self.training_context['grads_state']['last_layer'] = []
-
+                self.training_context['is_print_batch_progress']=is_print_batch_progress
+                self.training_context['is_print_epoch_progress'] = is_print_epoch_progress
                 self.training_context['print_batch_progress_frequency'] = 1
                 self.training_context['print_epoch_progress_frequency'] = 1
 
@@ -578,13 +592,19 @@ class ModelBase(object):
 
                 elif 'tensor' in output.__class__.__name__.lower():
                     train_data[self.outputs.key_list[0]] = output
+                    if self.use_output_as_loss==True:
+                        if 'self.outputs.key_list[0]' not in self.training_context['losses']:
+                            self.training_context['losses'][self.outputs.key_list[0]] = []
+                        this_loss=output.sum()
+                        self.training_context['losses'][self.outputs.key_list[0]].append(this_loss)
+                        self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
                 else:
                     train_data[self.outputs.key_list[0]] = output
                     if self.use_output_as_loss==True:
                         if 'self.outputs.key_list[0]' not in self.training_context['losses']:
                             self.training_context['losses'][self.outputs.key_list[0]] = []
                         this_loss=output.sum()
-                        self.training_context['losses'][self.outputs.key_list[0]]=this_loss
+                        self.training_context['losses'][self.outputs.key_list[0]].append(this_loss)
                         self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
             except Exception as e:
                 print(e)
@@ -592,7 +612,7 @@ class ModelBase(object):
                 if isinstance(self._model, Layer) and any_abnormal_number(self._model):
                     for para in self._model.parameters():
                         if any_abnormal_number(para):
-                            where(is_nan(para).all(), random_normal_like(para, mean=0, std=0.02).to(get_device()), para)
+                            para.data.copy_(where(is_nan(para), random_normal_like(para, mean=0, std=0.02).to(get_device()), para))
 
             # write output in to data
 
@@ -625,7 +645,8 @@ class ModelBase(object):
                                 else:
                                     # a leaf Variable that requires grad connotused in an in-place operation.
                                     overall_loss =overall_loss+ this_loss[i]
-                                    self.training_context['current_loss'] =self.training_context['current_loss']+ overall_loss
+                            self.training_context['current_loss'] =self.training_context['current_loss']+ overall_loss
+
                             if is_collect_data:
                                 self.training_context['losses'][k].append(float(to_numpy(overall_loss)))
                         else:
@@ -680,7 +701,7 @@ class ModelBase(object):
                 if isinstance(self._model, Layer) and any_abnormal_number(self._model):
                     for para in self._model.parameters():
                         if any_abnormal_number(para):
-                            where(is_nan(para).all(), random_normal_like(para, mean=0, std=0.02).to(get_device()), para)
+                            para.data.copy_(where(is_nan(para), random_normal_like(para, mean=0, std=0.02).to(get_device()), para))
 
                 # model comfirm
                 for k, v in self._constraints.items():
@@ -715,7 +736,7 @@ class ModelBase(object):
 
 
                 for k, v in self._metrics.items():
-                    collect_history = getattr(v,'collect_history')
+                    collect_history =getattr(v,'collect_history') if  hasattr(v,'collect_history') else True
                     if k not in self.training_context['metrics'] :
                         self.training_context['tmp_metrics'][k] = []
                         self.training_context['metrics'][k] = []
@@ -741,15 +762,13 @@ class ModelBase(object):
 
                 #callback's metric can keep in epoch_metric_history
                 for k, v in self.training_context['tmp_metrics'].items():
-                    if not getattr(self._metrics[k], 'collect_history') ==False:
-                        if k not in self.epoch_metric_history:
+                    if k not in self.epoch_metric_history:
                             self.epoch_metric_history[k] = []
 
                 if is_collect_data:
                     for k, v in self.training_context['tmp_metrics'].items():
-                        if not getattr(self._metrics[k], 'collect_history')== False:
-                            self.training_context['metrics'][k].append(float(to_numpy(v).mean()))
-                            self.training_context['tmp_metrics'][k] = []
+                        self.training_context['metrics'][k].append(float(to_numpy(v).mean()))
+                        self.training_context['tmp_metrics'][k] = []
 
                 # print batch progresss
                 if is_print_batch_progress:
@@ -812,7 +831,7 @@ class ModelBase(object):
         raise NotImplementedError
 
     def predict(self,input):
-        if isinstance(input,(torch.Tensor,np.ndarray)):
+        if isinstance(input,(Tensor,np.ndarray)):
             if isinstance(input,np.ndarray):
                 input=to_tensor(input)
             if len(input.shape)==len(self.inputs.value_list[0])+1:
