@@ -12,45 +12,66 @@ import uuid
 import weakref
 from collections import defaultdict, namedtuple
 from itertools import islice
-from typing import List
+from typing import List, Callable, TypeVar, Union, Tuple, overload, Mapping, Dict
 
 import numpy as np
-
+from distutils.version import Version, LooseVersion
 import tensorflow as tf
 from tensorflow.python import enable_eager_execution
 from tensorflow.python.eager import context
 from tensorflow.python.framework import func_graph, ops
+from tensorflow.python.training.tracking import base as trackable
+from tensorflow.python.training.tracking import data_structures,tracking
+from tensorflow.python.training.tracking import layer_utils as trackable_layer_utils
 
 from tensorflow.python.module import module
 from tensorflow.python.util import object_identity
-
+from tensorflow.python.keras.engine import base_layer_utils
 from trident.backend import tensorflow_serialization as serialization
-from trident.backend.common import camel2snake, to_list, unpack_singleton, enforce_singleton, OrderedDict, get_signature
+from trident.backend.common import camel2snake, to_list, unpack_singleton, enforce_singleton, OrderedDict, get_signature,get_session,set_session
 from trident.backend.tensorflow_ops import *
 from trident.data.utils import pickle_it
 
+
 __all__ = ['Layer','get_device', 'get_flops', 'Sequential', 'ReplayBuffer', 'summary', 'normalize_padding', 'load', 'save','try_map_args_and_call']
 
-gpus = tf.config.list_physical_devices('GPU')
-if gpus is not None and len(gpus)>0:
-    tf.config.experimental.set_memory_growth(gpus[0], True)
 
 def get_device():
-    if gpus:
-        return gpus[0]
-    else:
-        return "/cpu:0"
+    """get current device
+
+    Returns: device string ('cpu', 'cuda)
+
+    """
+    if get_session().device is None:
+        set_device('/gpu:0' if  len(tf.config.list_physical_devices('GPU'))>0 else "/cpu:0")
+    return get_session().device
+
+
+def set_device(device='/cpu:0'):
+    if device.lower()=='cuda' or device.lower()=='gpu':
+        device='/gpu:0'
+    if device.lower()=='cpu':
+        device='/cpu:0'
+    if 'gpu' in device and len(tf.config.list_physical_devices('GPU'))==0:
+        raise ValueError('Gpu is not available...')
+    try:
+        set_session('device', device)
+        tf.device(device)
+    except Exception as e:
+        print(e)
+
+
 
 
 version = tf.version
 sys.stdout.write('Tensorflow version:{0}.\n'.format(version.VERSION))
 
-#
-# tf_version=LooseVersion(vstring=version.VERSION)
-# base_version=LooseVersion(vstring='2.2.0-rc0')
-#
-# if tf_version.version <base_version.version:
-#     raise ValueError('trident only support Tensorflow 2.2.0-rc0 or newer.\n')
+
+tf_version=LooseVersion(vstring=version.VERSION)
+base_version=LooseVersion(vstring='2.2.0-rc0')
+
+if tf_version.version <base_version.version:
+    raise ValueError('trident only support Tensorflow 2.2.0-rc0 or newer.\n')
 
 sys.stdout.write('use device:{0}.\n'.format(get_device()))
 
@@ -63,12 +84,22 @@ except Exception as e:
 
 
 def load(path):
-    item = serialization.load(path)
-    return item
+    """load model from *.pth or *.pth.tar
+
+    Args:
+        path (str):
+
+    Returns:
+
+    """
+    if '.tar' in path:
+        return serialization.load_pthtar(path)
+    else:
+        return  serialization.load(path)
 
 
 def save(obj, path, is_compressed=False):
-    serialization.save(obj, path, _use_new_zipfile_serialization=is_compressed)
+    serialization.save(obj, path, is_compressed=is_compressed)
     return True
 
 
@@ -193,6 +224,122 @@ def _addindent(s_, numSpaces):
 
 
 
+r"""This tracks hooks common to all modules that are executed before/after
+calling forward and backward. This is global state used for debugging/profiling
+purposes"""
+_global_backward_hooks = OrderedDict()
+_global_forward_pre_hooks = OrderedDict()
+_global_forward_hooks = OrderedDict()
+
+_grad_t = Union[Tuple[Tensor, ...], Tensor]
+# See https://mypy.readthedocs.io/en/latest/generics.html#generic-methods-and-generic-self for the use
+# of `T` to annotate `self`. Many methods of `Module` return `self` and we want those return values to be
+# the type of the subclass, not the looser type of `Module`.
+T = TypeVar('T', bound='Layer')
+
+def register_module_forward_pre_hook(hook: Callable[..., None]) -> RemovableHandle:
+    r"""Registers a forward pre-hook common to all modules.
+
+    .. warning ::
+
+        This adds global state to the `nn.module` module
+        and it is only intended for debugging/profiling purposes.
+
+    The hook will be called every time before :func:`forward` is invoked.
+    It should have the following signature::
+
+        hook(module, input) -> None or modified input
+
+    The input contains only the positional arguments given to the module.
+    Keyword arguments won't be passed to the hooks and only to the ``forward``.
+    The hook can modify the input. User can either return a tuple or a
+    single modified value in the hook. We will wrap the value into a tuple
+    if a single value is returned(unless that value is already a tuple).
+
+    This hook has precedence over the specific module hooks registered with
+    ``register_forward_pre_hook``.
+
+    Returns:
+        :class:`torch.utils.hooks.RemovableHandle`:
+            a handle that can be used to remove the added hook by calling
+            ``handle.remove()``
+    """
+    handle = RemovableHandle(_global_forward_pre_hooks)
+    _global_forward_pre_hooks[handle.id] = hook
+    return handle
+
+
+def register_module_forward_hook(hook: Callable[..., None]) -> RemovableHandle:
+    r"""Registers a global forward hook for all the modules
+
+    .. warning ::
+
+        This adds global state to the `nn.module` module
+        and it is only intended for debugging/profiling purposes.
+
+    The hook will be called every time after :func:`forward` has computed an output.
+    It should have the following signature::
+
+        hook(module, input, output) -> None or modified output
+
+    The input contains only the positional arguments given to the module.
+    Keyword arguments won't be passed to the hooks and only to the ``forward``.
+    The hook can modify the output. It can modify the input inplace but
+    it will not have effect on forward since this is called after
+    :func:`forward` is called.
+
+    Returns:
+        :class:`torch.utils.hooks.RemovableHandle`:
+            a handle that can be used to remove the added hook by calling
+            ``handle.remove()``
+
+    This hook will be executed before specific module hooks registered with
+    ``register_forward_hook``.
+    """
+    handle =RemovableHandle(_global_forward_hooks)
+    _global_forward_hooks[handle.id] = hook
+    return handle
+
+def register_module_backward_hook(
+    hook: Callable[['Layer', _grad_t, _grad_t], Union[None, Tensor]]
+) -> RemovableHandle:
+    r"""Registers a backward hook common to all the modules.
+
+    .. warning ::
+        This adds global state to the `nn.module` module
+        and it is only intended for debugging/profiling purposes.
+
+        The current implementation will not have the presented behavior
+        for complex :class:`Module` that perform many operations.
+        In some failure cases, :attr:`grad_input` and :attr:`grad_output` will only
+        contain the gradients for a subset of the inputs and outputs.
+        For such :class:`Module`, you should use :func:`torch.Tensor.register_hook`
+        directly on a specific input or output to get the required gradients.
+
+    The hook will be called every time the gradients with respect to module
+    inputs are computed. The hook should have the following signature::
+
+        hook(module, grad_input, grad_output) -> Tensor or None
+
+    The :attr:`grad_input` and :attr:`grad_output` may be tuples if the
+    module has multiple inputs or outputs. The hook should not modify its
+    arguments, but it can optionally return a new gradient with respect to
+    input that will be used in place of :attr:`grad_input` in subsequent
+    computations. :attr:`grad_input` will only correspond to the inputs given
+    as positional arguments.
+
+    Global hooks are called before hooks registered with `register_backward_hook`
+
+    Returns:
+        :class:`torch.utils.hooks.RemovableHandle`:
+            a handle that can be used to remove the added hook by calling
+            ``handle.remove()``
+
+    """
+    handle =RemovableHandle(_global_backward_hooks)
+    _global_backward_hooks[handle.id] = hook
+    return handle
+
 
 
 
@@ -250,51 +397,65 @@ class Layer(tf.Module):
 
 
         """
-        super(Layer, self).__init__()
-        self.training = True
-        self._built = False
-        self.rank = kwargs.get('rank', None)
-
         self._uid_prefixs = {}
-        self._modules = OrderedDict()
-        self._parameters = OrderedDict()
-        self._buffers = OrderedDict()
-        self._backward_hooks = OrderedDict()
-        self._forward_hooks = OrderedDict()
-        self._forward_pre_hooks = OrderedDict()
-        self._state_dict_hooks = OrderedDict()
-        self._load_state_dict_pre_hooks = OrderedDict()
-
-        self._non_persistent_buffers_set = set()
-
-        self._input_shape = None
-        self.input_filters = None
-        self._output_shape = None
-        self.keep_output = keep_output
-        self._output_tensor = None
-        self.signature = None
-
         self._name = kwargs.get('name', name)
         self.uuid = uuid.uuid4().node
         prefix = self.__class__.__name__
         self.default_name = camel2snake(prefix) + '_' + str(get_global_uid(camel2snake(prefix)))
-        reset_name(self, self._uid_prefixs)
+        #reset_name(self, self._uid_prefixs)
         self.relative_name = ''
 
+        super(Layer, self).__init__(name=self.default_name )
 
-        # self.dump_patches = True
         self._nodes = None
-        self._device = '/cpu:0' if gpus is None or len(gpus) == 0 else gpus[0]
+
+        with self.name_scope:
+            self.batch_index = 0
+            self.filter_index = -1
+            self.in_sequence = False
+            self.training = True
+            self._built = False
+            self.rank = kwargs.get('rank', None)
 
 
-    @property
-    def name(self):
-        """If not assign name , it will return the default_name"""
-        return self._name if self._name is not None and len(self._name) > 0 else self.default_name
 
-    @name.setter
-    def name(self, value):
-        self._name = value
+            self._modules = OrderedDict()
+            self._parameters = OrderedDict()
+            self._buffers = OrderedDict()
+            self._backward_hooks = OrderedDict()
+            self._forward_hooks = OrderedDict()
+            self._forward_pre_hooks = OrderedDict()
+            self._state_dict_hooks = OrderedDict()
+            self._load_state_dict_pre_hooks = OrderedDict()
+
+            self._non_persistent_buffers_set = set()
+
+            self._input_shape = None
+            self.input_filters = None
+            self._output_shape = None
+            self.keep_output = keep_output
+            self._output_tensor = None
+            self.signature = None
+
+
+
+
+            # self.dump_patches = True
+
+            self._device = get_device()
+
+
+    # @property
+    # def name(self):
+    #     """If not assign name , it will return the default_name"""
+    #     return self._name if self._name is not None and len(self._name) > 0 else self.default_name
+    #
+    # @name.setter
+    # def name(self, value):
+    #     self._name = value
+
+    # def _set_name_scope(self):
+
 
     @property
     def nodes(self):
@@ -308,7 +469,6 @@ class Layer(tf.Module):
             for mod in self.modules():
                 mod._nodes = value
 
-
     def add_module(self, name, module):
         """Adds a child module to the current module.
 
@@ -379,6 +539,7 @@ class Layer(tf.Module):
         else:
             raise ValueError('Not valid module')
 
+
     def build(self, input_shape):
         """ Do the shape inference and initialize weights and bias.
 
@@ -505,11 +666,12 @@ class Layer(tf.Module):
             raise TypeError("cannot assign '{}' object to buffer '{}' "
                             "(torch Tensor or None required)".format(type(tensor).__name__, name))
         else:
-            self._buffers[name] = tensor
-            if persistent:
-                self._non_persistent_buffers_set.discard(name)
-            else:
-                self._non_persistent_buffers_set.add(name)
+            with self.name_scope:
+                self._buffers[name] = tensor
+                if persistent:
+                    self._non_persistent_buffers_set.discard(name)
+                else:
+                    self._non_persistent_buffers_set.add(name)
 
     def register_parameter(self, name, param):
         r"""Adds a parameter to the module.
@@ -522,6 +684,7 @@ class Layer(tf.Module):
             param (Parameter): parameter to be added to the module.
 
         """
+
         if '_parameters' not in self.__dict__:
             raise AttributeError("cannot assign parameter before Module.__init__() call")
 
@@ -541,16 +704,9 @@ class Layer(tf.Module):
             raise TypeError("cannot assign '{}' object to parameter '{}' "
                             "(tf.Variable or None required)".format(type(param).__name__, name))
         else:
-
             self._parameters[name] = param
 
 
-
-
-
-
-
-
     def cuda(self, device=None):
         r"""Moves all model parameters and buffers to the GPU.
 
@@ -565,7 +721,11 @@ class Layer(tf.Module):
         Returns:
             Module: self
         """
-        self._device = '/cpu:0'
+        if tf.test.is_gpu_available:
+            self._device = '/gpu:0'
+            tf.device(self._device)
+        else:
+            sys.stderr.write('GPU is not available in this machone./n')
 
     def cpu(self):
         r"""Moves all model parameters and buffers to the CPU.
@@ -616,7 +776,7 @@ class Layer(tf.Module):
 
     @property
     def device(self):
-        return tf.device()
+        return self._device
 
     @property
     def built(self):
@@ -644,9 +804,16 @@ class Layer(tf.Module):
         if self._built == False:
             self._input_shape = value
             if len(self._input_shape) == 0:
-                self.input_filters =  int(self._input_shape)
+                self.input_filters = int(to_numpy(self._input_shape).data)
+            elif len(self._input_shape) == 1:
+                self.input_filters = int(to_numpy(self._input_shape)[-1])
             else:
-                self.input_filters = int(self._input_shape[-1])
+                if self.filter_index < 0:
+                    self.input_filters = int(self._input_shape[self.filter_index])
+                elif self.filter_index > self.batch_index:
+                    self.input_filters = int(self._input_shape[self.filter_index - self.batch_index - 1])
+                else:
+                    raise NotImplementedError('filter_index>batch_index')
 
             self.build(self._input_shape)
             self._built = True
@@ -702,6 +869,20 @@ class Layer(tf.Module):
     def copy(self):
         return copy.deepcopy(self)
 
+        # The user can pass an optional arbitrary mappable object to `state_dict`, in which case `state_dict` returns
+        # back that same object. But if they pass nothing, an `OrederedDict` is created and returned.
+
+    T_destination = TypeVar('T_destination', bound=Mapping[str, Tensor])
+
+    @overload
+    def state_dict(self, destination: T_destination, prefix: str = ..., keep_vars: bool = ...) -> T_destination:
+        ...
+
+    # TODO: annotate with OrderedDict not Dict, but there is a problem:
+    # https://docs.python.org/3/library/typing.html#typing.OrderedDict
+    @overload
+    def state_dict(self, prefix: str = ..., keep_vars: bool = ...) -> Dict[str, Tensor]:
+        ...
     def _save_to_state_dict(self, destination, prefix, keep_vars):
         r"""Saves module state to `destination` dictionary, containing a state
         of the module, but not its descendants. This is called on every
@@ -731,10 +912,12 @@ class Layer(tf.Module):
             >>> module.state_dict().keys()
             ['bias', 'weight']
         """
+
         if destination is None:
             destination = OrderedDict()
             destination._metadata = OrderedDict()
         destination._metadata[prefix[:-1]] = local_metadata = dict(version=self._version)
+
         self._save_to_state_dict(destination, prefix, keep_vars)
         for name, module in self._modules.items():
             if module is not None:
@@ -755,8 +938,9 @@ class Layer(tf.Module):
         self._load_state_dict_pre_hooks[handle.id] = hook
         return handle
 
-    def _load_from_state_dict(self, state_dict, prefix, local_metadata, strict, missing_keys, unexpected_keys,
-                              error_msgs):
+
+    def _load_from_state_dict(self, state_dict, prefix, local_metadata, strict,
+                              missing_keys, unexpected_keys, error_msgs):
         r"""Copies parameters and buffers from :attr:`state_dict` into only
         this module, but not its descendants. This is called on every submodule
         in :meth:`~torch.nn.Module.load_state_dict`. Metadata saved for this
@@ -764,11 +948,13 @@ class Layer(tf.Module):
         For state dicts without metadata, :attr:`local_metadata` is empty.
         Subclasses can achieve class-specific backward compatible loading using
         the version number at `local_metadata.get("version", None)`.
+
         .. note::
             :attr:`state_dict` is not the same object as the input
             :attr:`state_dict` to :meth:`~torch.nn.Module.load_state_dict`. So
             it can be modified.
-        Args:
+
+        Arguments:
             state_dict (dict): a dict containing parameters and
                 persistent buffers.
             prefix (str): the prefix for parameters and buffers used in this
@@ -789,7 +975,8 @@ class Layer(tf.Module):
         for hook in self._load_state_dict_pre_hooks.values():
             hook(state_dict, prefix, local_metadata, strict, missing_keys, unexpected_keys, error_msgs)
 
-        local_name_params = itertools.chain(self._parameters.items(), self._buffers.items())
+        persistent_buffers = {k: v for k, v in self._buffers.items() if k not in self._non_persistent_buffers_set}
+        local_name_params = itertools.chain(self._parameters.items(), persistent_buffers.items())
         local_state = {k: v for k, v in local_name_params if v is not None}
 
         for name, param in local_state.items():
@@ -804,17 +991,19 @@ class Layer(tf.Module):
                 if input_param.shape != param.shape:
                     # local shape should match the one in checkpoint
                     error_msgs.append('size mismatch for {}: copying a param with shape {} from checkpoint, '
-                                      'the shape in current model is {}.'.format(key, input_param.shape, param.shape))
+                                      'the shape in current model is {}.'
+                                      .format(key, input_param.shape, param.shape))
                     continue
 
                 try:
-                    setattr(self, name, tf.Variable(to_numpy(input_param)))  # param=input_param
+                    param.assign(to_tensor(input_param))
 
                 except Exception as ex:
                     error_msgs.append('While copying the parameter named "{}", '
                                       'whose dimensions in the model are {} and '
                                       'whose dimensions in the checkpoint are {}, '
-                                      'an exception occured : {}.'.format(key, param.shape, input_param.shape, ex.args))
+                                      'an exception occured : {}.'
+                                      .format(key, param.size(), input_param.size(), ex.args))
             elif strict:
                 missing_keys.append(key)
 
@@ -923,6 +1112,7 @@ class Layer(tf.Module):
 
     @tf.Module.with_name_scope
     def __call__(self, *input, **kwargs):
+        # Maintains info about the `Layer.call` stack.
         is_all_numpy=True
         input = list(input)
         new_input = []
@@ -943,22 +1133,23 @@ class Layer(tf.Module):
         if self._built==False :
             inp= unpack_singleton(input)
             if isinstance(inp, (tuple, list)):
-                self.build([input_tensor.shape[1:] for input_tensor in inp])
+                self.build([input_tensor.shape[self.batch_index+1:] for input_tensor in inp])
             elif is_tensor(inp):
-                self.input_shape = inp.shape[1:]
+                self.input_shape = inp.shape[self.batch_index+1:]
             else:
                 print('input shou be tensor or tuple of tensor')
                 print(inp)
                 self.input_shape= tf.TensorShape(None)
         # don't use result = self.forward(i*nput, **kwargs) because EagerTensor will splited as a tuple....
         try:
-            result = self.forward(*input, **kwargs)
-            output = unpack_singleton(result)
-            if hasattr(self, 'keep_output') and self.keep_output == True:
-                self._output_tensor = output
-            if is_tensor(output):
-                if self._output_shape is None:
-                    self.output_shape = output.shape[1:]
+            with tf.device(get_device()):
+                result = self.forward(*input, **kwargs)
+                output = unpack_singleton(result)
+                if hasattr(self, 'keep_output') and self.keep_output == True:
+                    self._output_tensor = output
+                if is_tensor(output):
+                    if self._output_shape is None:
+                        self.output_shape = output.shape[self.batch_index+1:]
 
             for hook in self._forward_hooks.values():
                 hook_result = hook(self, input, result)
@@ -1422,6 +1613,19 @@ class Layer(tf.Module):
 
         return sorted(keys)
 
+    def _replicate_for_data_parallel(self):
+        replica = self.__new__(type(self))
+        replica.__dict__ = self.__dict__.copy()
+
+        # replicas do not have parameters themselves, the replicas reference the original
+        # module.
+        replica._parameters = OrderedDict()
+        replica._buffers = replica._buffers.copy()
+        replica._modules = replica._modules.copy()
+        replica._is_replica = True
+
+        return replica
+
 
 class Sequential(Layer):
     r"""A sequential container.
@@ -1452,7 +1656,7 @@ class Sequential(Layer):
         self._built = False
         if len(args) == 1 and isinstance(args[0], OrderedDict):
             for key, module in args[0].items():
-                module.name = key
+                module._name = key
                 self.add_module(key, module)
         elif len(args) == 1 and isinstance(args[0], list):
             for idx, module in enumerate(args[0]):
@@ -1834,18 +2038,31 @@ def normalize_padding(padding, rank):
     return padding
 
 
-def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
+def try_map_args_and_call(fn, data: OrderedDict, data_feed=None,grad_tape=None):
+    """This function is the core function for mapping callable and argments
+
+    Args:
+        fn (callable): the callable, maybe functions or layers
+        data (OrderedDict): The key-value pair for available data.
+        data_feed (OrderedDict): The relation between callable argments (key) and data (value)
+        grad_tape (tf.GradientTape):
+
+    Returns:
+        The result of the callable base on data_feed
+
+    """
+
     if isinstance(fn, tf.Tensor) or 'EagerTensor' in fn.__class__.__name__:
         return fn
     else:
+
         arg_map = OrderedDict()
         if isinstance(fn, Layer):
             for arg in fn.signature.inputs.key_list:
-                if arg in data:
-                    arg_map[arg] = data[arg]
-                elif arg in data_feed:
+                if arg in data_feed:
                     arg_map[arg] = data[data_feed[arg]]
-
+                elif arg in data:
+                    arg_map[arg] = data[arg]
                 else:
                     raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
             # print('arg_map',arg_map.key_list)
@@ -1858,10 +2075,11 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
                 return out
         elif hasattr(fn, 'signature') and callable(fn):
             for arg in fn.signature.inputs.key_list:
-                if arg in data:
-                    arg_map[arg] = data[arg]
-                elif arg in data_feed:
+                if arg in data_feed:
                     arg_map[arg] = data[data_feed[arg]]
+                elif arg in data:
+                    arg_map[arg] = data[arg]
+
 
                 elif arg == 'y_pred' and 'output' in data:
                     arg_map[arg] = data['output']
@@ -1889,10 +2107,13 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
             out = fn(*arg_map.value_list)
             return out
         else:
-
             print('uncomplete arg_map', arg_map.key_list)
 
 
+
+
+
+
 def force_deterministic(seed):
     """ Force most of the computation nodes to run deterministically.
 
