@@ -20,12 +20,12 @@ from trident.data.bbox_common  import *
 
 if get_backend()=='pytorch':
     from trident.backend.pytorch_backend import try_map_args_and_call
-    from trident.backend.pytorch_ops import to_numpy,to_tensor,arange,shuffle,cast,clip,sqrt,int_shape,argmax,softmax
+    from trident.backend.pytorch_ops import to_numpy,to_tensor,arange,shuffle,cast,clip,sqrt,int_shape,argmax,softmax,any_abnormal_number
 
 elif get_backend()=='tensorflow':
     from trident.backend.tensorflow_backend import try_map_args_and_call
-    from trident.backend.tensorflow_ops import  to_numpy,to_tensor,arange,shuffle,cast,clip,sqrt,int_shape,concate,zeros_like,ones_like,argmax,softmax
-
+    from trident.backend.tensorflow_ops import to_numpy, to_tensor, arange, shuffle, cast, clip, sqrt, int_shape, concate, zeros_like, ones_like, argmax, softmax, any_abnormal_number, \
+    not_equal
 
 if is_in_ipython() or is_in_colab():
     from IPython import display
@@ -91,11 +91,15 @@ class TileImageCallback(VisualizationCallbackBase):
 
         else:
             for data_key in data.key_list:
-                if data_key == data_feed[model.signature.key_list[0]]:
-                    input = data[data_feed[model.signature.key_list[0]]]
-                elif data_key=='output':
-                    output = data['output']
-                elif data_key=='target':
+                if  self.include_input  and data_key == model.signature.inputs.key_list[0]:
+                    input = data[data_key]
+                elif self.include_output and data_key== model.signature.outputs.key_list[0]:
+                    output = data[data_key]
+                elif  self.include_input  and model.signature.inputs.key_list[0] in data_feed and data_feed[model.signature.inputs.key_list[0]]==data_key:
+                    input = data[data_key]
+                elif self.include_output  and model.signature.outputs.key_list[0] in data_feed and data_feed[model.signature.outputs.key_list[0]]==data_key:
+                    output = data[data_key]
+                elif self.include_target and data_key=='target':
                     output = data['target']
                 elif 'output' in data_key or 'pred' in data_key:
                         output = data[data_key]
@@ -171,8 +175,8 @@ class SegTileImageCallback(VisualizationCallbackBase):
             is_label_mask = True
         # if len(data) >= 3:
         for data_key in data.key_list:
-            if data_key == data_feed[model.signature.key_list[0]]:
-                input = data[data_feed[model.signature.key_list[0]]]
+            if data_key == data_feed[model.signature.inputs.key_list[0]]:
+                input = data[data_feed[model.signature.inputs.key_list[0]]]
 
                 training_context['current_model'].eval()
                 output = model(input)
@@ -347,7 +351,7 @@ class PlotLossMetricsCallback(VisualizationCallbackBase):
 
 
 class PrintGradientsCallback(VisualizationCallbackBase):
-    def __init__(self, batch_inteval=-1):
+    def __init__(self, batch_inteval=100):
         super(PrintGradientsCallback, self).__init__(epoch_inteval=-1, batch_inteval=batch_inteval)
         self.is_in_ipython = is_in_ipython()
         self.is_in_colab = is_in_colab()
@@ -398,12 +402,12 @@ class PrintGradientsCallback(VisualizationCallbackBase):
                         training_context['grads_state']['last_layer'][-1]))
         elif get_backend() == 'tensorflow':
             if  (training_context['current_epoch'] * training_context['total_batch'] + training_context['current_batch']) % self.batch_inteval == 0:
-                grad_dict = {}
                 if 'grads_state' not in training_context:
                     training_context['grads_state'] = OrderedDict()
                     training_context['grads_state']['first_layer'] = []
                     training_context['grads_state']['last_layer'] = []
-                grads_and_vars=training_context['optimizer'].grads_and_vars
+                grads_and_vars=list(training_context['optimizer'].grads_and_vars)
+                grads_and_vars=[gv for gv in grads_and_vars if gv[1].trainable and tf.reduce_any(tf.not_equal(gv[0],0))]
                 training_context['grads_state']['first_layer'].append( np.abs(to_numpy(grads_and_vars[0][0])).mean())
                 training_context['grads_state']['last_layer'].append(np.abs(to_numpy(grads_and_vars[-1][0])).mean())
 
@@ -412,6 +416,7 @@ class PrintGradientsCallback(VisualizationCallbackBase):
                     self.lines.append('{0:<16s}  first_layer gradients: {1:<8.3e}| last_layer gradients: {2:<8.3e}'.format(
                         training_context['current_model'].name, training_context['grads_state']['first_layer'][-1],
                         training_context['grads_state']['last_layer'][-1]))
+
     def on_overall_batch_end(self, training_context):
         if len(self.lines) > 0:
             sys.stdout.writelines(self.lines)
