@@ -4,14 +4,14 @@ from __future__ import division
 from __future__ import print_function
 
 import pickle
-
+import io
 import numpy as np
-
+import copy
 from trident.data.data_provider import *
 from trident.data.dataset import *
 from trident.data.mask_common import *
 from trident.data.utils import *
-from trident.backend.common import floatx
+from trident.backend.common import floatx,ExpectDataType,OrderedDict
 from trident.misc.ipython_utils import *
 
 try:
@@ -242,34 +242,73 @@ def load_birdsnap(dataset_name='birdsnap', kind='train', is_flatten=None, is_one
     return (images, labels)
 
 
-def load_text(filname, delimiter=',', skiprows=0, label_index=None, is_onehot=None, shuffle=True):
-    if _backend in ['tensorflow', 'cntk'] and is_onehot is None:
-        is_onehot = True
-    arr = np.genfromtxt(filname, delimiter=delimiter, skip_header=skiprows, dtype=floatx(), filling_values=0,
-                        autostrip=True)
-    data, labels = None, None
-    if label_index is None:
-        data = arr
-    else:
-        if label_index == 0:
-            data, labels = arr[:, 1:], arr[:, 0:1]
-        elif label_index == -1 or label_index == len(arr) - 1:
-            data, labels = arr[:, :-1], arr[:, -1:]
-        else:
-            rdata, labels = np.concatenate([arr[:, :label_index], arr[:, label_index + 1:]], axis=0), arr[:,
-                                                                                                      label_index:label_index + 1]
-    labels = np.squeeze(labels)
-    if _backend == 'pytorch':
-        labels = np.squeeze(labels).astype(np.int64)
-    if is_onehot == True:
-        if _backend == 'pytorch':
-            warnings.warn('Pytorch not prefer onehot label, are you still want onehot label?', category='data loading',
-                          stacklevel=1, source='load_text')
-        labels = to_onehot(labels)
-    idxes = np.arange(len(data))
-    dataset = DataProvider(filname.split('/')[-1].strip().split('.')[0], data=data, labels=labels, scenario='train')
+def load_text(filname=None, data=None, label=None,unit='char',mode='next_word',section_delimiter='\n\n',sequence_start_at='random',is_onehot=False,encoding='utf-8-sig',sequence_length=64, return_corpus=False,**kwargs):
+    valid_sequence_start_at=['random','slide','follow_up','section_start']
+    valid_mode=['next_word','skip_gram', 'cbow','onehot','1to1_seq2seq']
+    if mode not in valid_mode:
+        raise  ValueError('{0} is not valid mode '.format(mode))
 
-    return dataset
+    original_corpus = None
+    corpus = None
+    output_corpus=None
+    if filname is not None:
+        with io.open(filname, encoding=encoding) as f:
+            original_corpus = f.read().lower()
+            if unit == 'char':
+                corpus = list(original_corpus)
+            elif unit == 'word':
+                corpus = original_corpus.split(' \t')
+    if data is not None:
+        if isinstance(data,str):
+            if unit == 'char':
+                corpus = list(data)
+            elif unit == 'word':
+                corpus = data.split(' \t')
+        elif hasattr(data,"__iter__"):
+            corpus= '\n\n'.join(data)
+            if unit == 'char':
+                corpus = list(corpus)
+            elif unit == 'word':
+                corpus = corpus.replace('\n\n',' \n \n ').split(' \t')
+    if label is not None:
+        if isinstance(label,str):
+            if unit == 'char':
+                output_corpus = list(label)
+            elif unit == 'word':
+                output_corpus = label.split(' \t')
+        elif hasattr(label,"__iter__"):
+            output_corpus = '\n\n'.join(label)
+            if unit == 'char':
+                output_corpus = list(output_corpus)
+            elif unit == 'word':
+                output_corpus = output_corpus.replace('\n\n',' \n \n ').split(' \t')
+    dataprovider=None
+    if mode=='next_word':
+        corpus1=copy.deepcopy(corpus)
+        corpus2= copy.deepcopy(corpus)
+        data_seq=TextSequenceDataset(corpus1,sequence_length=sequence_length,is_onehot=is_onehot,symbol='input',sequence_offset=0,sequence_start_at=sequence_start_at)
+        labels_seq =TextSequenceDataset(corpus2,sequence_length=sequence_length,is_onehot=is_onehot,symbol='label',sequence_offset=1,sequence_start_at=sequence_start_at)
+        traindata=Iterator(data=data_seq,label=labels_seq)
+        dataprovider = TextSequenceDataProvider(filname.split('/')[-1].strip().split('.')[0],traindata=traindata)
+    elif mode=='skip_gram':
+        corpus1 = copy.deepcopy(corpus)
+        corpus2 = copy.deepcopy(corpus)
+        data_seq = TextSequenceDataset(corpus1, sequence_length=sequence_length, is_onehot=is_onehot, symbol='input', sequence_offset=0, sequence_start_at=sequence_start_at)
+        labels_seq = TextSequenceDataset(corpus2, sequence_length=sequence_length, is_onehot=is_onehot, symbol='label', sequence_offset=[-1,1], sequence_start_at=sequence_start_at)
+        traindata = Iterator(data=data_seq, label=labels_seq)
+        dataprovider = TextSequenceDataProvider(filname.split('/')[-1].strip().split('.')[0], traindata=traindata)
+    elif mode == '1to1_seq2seq':
+        if len(corpus)==len(output_corpus):
+            data_seq = TextSequenceDataset(corpus, sequence_length=sequence_length, is_onehot=is_onehot, symbol='input', sequence_offset=0, sequence_start_at=sequence_start_at)
+            labels_seq = TextSequenceDataset(output_corpus, sequence_length=sequence_length, is_onehot=is_onehot, symbol='label', sequence_offset=0, sequence_start_at=sequence_start_at)
+            traindata = Iterator(data=data_seq, label=labels_seq)
+            dataprovider = TextSequenceDataProvider('1to1_seq2seq', traindata=traindata)
+        else:
+            raise  ValueError('data ({0}) and label({1}) should have the same length in 1to1_seq2seq mide.'.format(len(corpus),len(output_corpus)))
+    if return_corpus:
+        return dataprovider,original_corpus
+    else:
+        return dataprovider
 
 
 def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=True, folder_as_label=True,
