@@ -3,7 +3,9 @@ from __future__ import division
 from __future__ import print_function
 import copy
 import hashlib
+import inspect
 import itertools
+import builtins
 import os
 import pickle
 import random
@@ -13,7 +15,6 @@ import threading
 import time
 from enum import Enum, unique
 from typing import List, TypeVar, Iterable, Tuple, Union
-
 import numpy as np
 from skimage import color
 
@@ -22,8 +23,9 @@ from trident.data.image_common import gray_scale, image2array, mask2array, image
     unnormalize, array2image, ExpectDataType, GetImageMode
 from trident.data.label_common import label_backend_adaptive
 from trident.data.mask_common import mask_backend_adaptive, color2label
+from trident.data.text_common import text_backend_adaption,reverse_text_backend_adaption
 from trident.data.samplers import *
-from trident.backend.common import DataSpec, PrintException, OrderedDict, Signature
+from trident.backend.common import *
 from trident.backend.load_backend import get_backend
 
 try:
@@ -35,7 +37,7 @@ if get_backend() == 'pytorch':
     from trident.backend.pytorch_backend import to_numpy, to_tensor, ReplayBuffer
     import torch
 
-__all__ = ['Dataset', 'ImageDataset', 'MaskDataset', 'LabelDataset', 'BboxDataset', 'MultipleDataset', 'Iterator', 'MetricIterator',
+__all__ = ['Dataset', 'ImageDataset', 'MaskDataset','TextSequenceDataset', 'LabelDataset', 'BboxDataset', 'MultipleDataset', 'Iterator', 'MetricIterator',
            'NumpyDataset', 'RandomNoiseDataset']
 
 T = TypeVar('T', int, float, str, np.ndarray)
@@ -53,11 +55,17 @@ class Dataset(List):
     def __add__(self, other):
         if other is not None and hasattr(other, '__iter__'):
             for item in other:
-                if isinstance(item, (int, float, str, np.ndarray)):
+                if isinstance(item, (int, float, str, np.ndarray,list)):
                     super().append(item)
 
     def __len__(self):
         return super().__len__()
+    @property
+    def length(self):
+        return self.__len__()
+
+    def len(self):
+        return self.__len__()
 
 
 class MultipleDataset(List):
@@ -431,8 +439,199 @@ class RandomNoiseDataset(Dataset):
         return sys.maxsize
 
 
+
+
+class TextSequenceDataset(Dataset):
+    def __init__(self, corpus=None, is_onehot=False,sequence_offset=0,section_delimiter='\n\n',stopwords=None, sequence_length:int=64,sequence_start_at='random',symbol=None, name=''):
+        super().__init__(symbol=symbol, name=name)
+        self.sequence_start_at=sequence_start_at
+        if len(section_delimiter)==2:
+            self.section_delimiter=section_delimiter
+        else:
+            self.section_delimiter='\n\n'
+        self.vocabs=None
+        self.text2index=None
+        self.index2text = None
+        self.is_onehot=is_onehot
+
+        if hasattr(corpus,"__iter__"):
+            new_corpus=[]
+            section=[]
+            section.append('<start/>')
+            for i in range(len(corpus)):
+                item=corpus[i]
+                if  item=='\n' and  corpus[i - 1]!='\n' and len(section)>0:
+                    section.append('<end/>')
+                    if i<len(corpus)-1 and corpus[i + 1] == '\n':
+                        new_corpus.append(section)
+                        section = []
+                        section.append('<start/>')
+                    elif i<len(corpus)-1 :
+                        section.append('<start/>')
+                elif self.section_delimiter!='\n\n' and len(self.section_delimiter)==2 and len(section)>0and item==self.section_delimiter[0] and i<len(corpus)-1  and corpus[i-1]==self.section_delimiter[1]:
+                    section.append('<end/>')
+                    if i < len(corpus) - 1 and corpus[i + 1] == '\n':
+                        new_corpus.append(section)
+                        section=[]
+                        section.append('<start/>')
+                    elif i < len(corpus) - 1:
+                        section.append('<start/>')
+                elif  item=='\r' and corpus[i-1]=='\n':
+                    pass
+                elif  item=='\n':
+                    pass
+                else:
+                    section.append(item)
+                    if i == len(corpus) - 1:
+                        section.append('<end/>')
+            if len(section)>0:
+                new_corpus.append(section)
+
+            if self.sequence_start_at=='random':
+                for sect in new_corpus:
+                    self.__add__(sect)
+                    self.__add__('\n')
+                    self.__add__('\n')
+            else:
+                self.__add__(new_corpus)
+
+
+            chars = sorted(list(set(corpus)))
+
+            chars.insert(0, '<start/>')
+            chars.insert(1, '<end/>')
+            chars.insert(2, '<unknown/>')
+            chars.insert(3, '<pad/>')
+            print('total distinct chars:', len(chars))
+            self.vocabs=chars
+            self.text2index = dict((c, i) for i, c in enumerate(chars))
+            self.index2text = dict((i, c) for i, c in enumerate(chars))
+        else:
+            raise ValueError('corpus should be a collection.')
+
+
+        self.sequence_offset=sequence_offset
+        self.dtype = np.float32
+        self.text_transform_funcs = []
+        self.is_pair_process = False
+        self.sequence_length=sequence_length
+
+
+
+    def _get_item_by_idx(self, iterator, idx):
+        """Get the idx-th item of the iterator"""
+        size = len(self)
+        idx = idx.__index__()
+        if not -size <= idx < size:
+            raise IndexError('index {} is out of range'.format(idx))
+        idx %= size
+        return next(itertools.islice(iterator, idx, None))
+
+    def __getitem__(self, index: int):
+        sequencetext=None
+        seq_base=list(self.__iter__())
+        if isinstance(self.sequence_offset,int):
+            if self.sequence_start_at=='random':
+                sequencetext=seq_base[index+self.sequence_offset:builtins.min(index+self.sequence_offset+self.sequence_length,self.len())]
+            elif self.sequence_start_at=='section_start':
+                sectiontext=seq_base[index]
+                sequencetext=sectiontext[self.sequence_offset:builtins.min(self.sequence_offset+self.sequence_length,self.len())]
+        elif isinstance(self.sequence_offset,list):
+            sequencetext=[]
+            if self.sequence_start_at=='random':
+                for k in self.sequence_offset:
+                    if 0<=index+self.sequence_offset<len(seq_base):
+                        sequencetext.append(seq_base[index+self.sequence_offset])
+                    else:
+                        sequencetext.append('<pad/>')
+
+            elif self.sequence_start_at=='section_start':
+                sectiontext=seq_base[index]
+                for k in self.sequence_offset:
+                    if 0 <= index + self.sequence_offset < len(seq_base):
+                        sequencetext.append(seq_base[index + self.sequence_offset])
+                    else:
+                        sequencetext.append('<pad/>')
+
+        if len(sequencetext) != self.sequence_length:
+            sequencetext.extend(['<pad/>']*(self.sequence_length-len(sequencetext)))
+        arr=None
+        if self.is_onehot:
+            arr=np.zeros((self.sequence_length,len(self.text2index)))
+            for i in range(self.sequence_length):
+                this_char=sequencetext[i]
+                if i+1<self.sequence_length and  ''.join(sequencetext[i:i+2])=='\n\n' :
+                    arr[i:, self.text2index['<pad/>']] = 1
+                    break
+                elif this_char in self.text2index:
+                    arr[i,self.text2index[this_char]]=1
+                else:
+                    arr[i, self.text2index['<unknown/>']] = 1
+            arr = arr.astype(np.float32)
+        else:
+            arr = np.zeros((self.sequence_length))
+            for i in range(self.sequence_length):
+                this_char = sequencetext[i]
+                if i+1<self.sequence_length and ''.join(sequencetext[i:i+2])=='\n\n' :
+                    arr[i:] = self.text2index['<pad/>']
+                    break
+                elif this_char in self.text2index:
+                    arr[i] =self.text2index[this_char]
+                else:
+                    arr[i] =self.text2index['<unknown/>']
+            arr=arr.astype(np.int64)
+
+        if self.is_pair_process == False and len(self.text_transform_funcs)==0:
+            return text_backend_adaption(arr)
+        elif  self.is_pair_process == False:
+            return self.text_transform(arr)
+        elif self.is_pair_process == True:
+            return arr
+
+        return None
+
+    def text_transform(self, text_data):
+        if len(self.text_transform_funcs) == 0:
+            return text_backend_adaption(text_data)
+        if isinstance(text_data, np.ndarray):
+            for fc in self.text_transform_funcs:
+                if not fc.__qualname__.startswith(
+                        'random_') or 'crop' in fc.__qualname__ or 'rescale' in fc.__qualname__ or (
+                        fc.__qualname__.startswith('random_') and random.randint(0, 10) % 2 == 0):
+                    text_data = fc(text_data)
+            text_data = text_backend_adaption(text_data)
+
+            return text_data
+        else:
+            return text_data
+
+    @property
+    def reverse_text_transform_funcs(self):
+        return_list = []
+        return_list.append(reverse_text_backend_adaption)
+        for i in range(len(self.text_transform_funcs)):
+            fn = self.text_transform_funcs[-1 - i]
+            # if fn.__qualname__ == 'normalize.<locals>.text_op':
+            #     return_list.append(unnormalize(fn.mean, fn.std))
+        #return_list.append(array2image)
+        return return_list
+
+    def reverse_text_transform(self, text_data):
+        if len(self.reverse_text_transform_funcs) == 0:
+            return reverse_text_backend_adaption(text_data)
+        if isinstance(text_data, np.ndarray):
+            # if img_data.ndim>=2:
+            for fc in self.reverse_text_transform_funcs:
+                text_data = fc(text_data)
+            text_data = reverse_text_backend_adaption(text_data)
+
+            return text_data
+        else:
+            return text_data
+
+
 class Iterator(object):
-    def __init__(self, data=None, label=None, mask=None, unpair=None, minibatch_size=8):
+    def __init__(self, data=None, label=None, mask=None, unpair=None,sample_filter=None, minibatch_size=8):
         self.is_pair_process = False
         self.signature = None
         self._data = NumpyDataset()
@@ -440,12 +639,14 @@ class Iterator(object):
         self._unpair = NumpyDataset()
         self.workers = 2
         self.itr = 0
-        if data is not None and isinstance(data, (Dataset, MultipleDataset)):
+        if data is not None and isinstance(data, (Dataset, MultipleDataset,TextSequenceDataset)):
             self._data = data
-        if label is not None and isinstance(label, (Dataset, MultipleDataset)):
+        if label is not None and isinstance(label, (Dataset, MultipleDataset,TextSequenceDataset)):
             self._label = label
             if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset, MultipleDataset)) and isinstance(self._data, ImageDataset) and len(self._label) == len(self._data):
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
+            elif isinstance(self._label, TextSequenceDataset) and isinstance(self._data, TextSequenceDataset) and len(self._label) == len(self._data):
+                self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
             else:
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
 
@@ -458,6 +659,10 @@ class Iterator(object):
         self._sample_iter = iter(self.batch_sampler)
         self.buffer_size = 10
         self.out_queue = Queue.Queue(maxsize=self.buffer_size)
+        self.sample_filter = None
+        if inspect.isfunction(sample_filter) or callable(sample_filter) :
+            self.sample_filter = sample_filter
+            self.batch_sampler.sample_filter = self.sample_filter
 
     @property
     def data(self):
@@ -473,8 +678,10 @@ class Iterator(object):
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
 
         self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
+        self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
+
     @property
     def label(self):
         return self._label
@@ -487,7 +694,7 @@ class Iterator(object):
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
         else:
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
-
+        self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
     @property
@@ -497,6 +704,7 @@ class Iterator(object):
     @unpair.setter
     def unpair(self, value):
         self._unpair = value
+        self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
     @property
@@ -514,6 +722,7 @@ class Iterator(object):
     def minibatch_size(self, value):
         self._minibatch_size = value
         self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
+        self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
     def update_signature(self, arg_names):
@@ -581,8 +790,10 @@ class Iterator(object):
                         bbox, label = new_label
                     else:
                         bbox = new_label
+                        label=None
                 else:
                     bbox = label
+                    label = None
             elif isinstance(self.label, MaskDataset):
                 data, label = self.paired_transform(data, label)
                 if hasattr(self.data, 'image_transform'):
@@ -697,7 +908,7 @@ class MetricIterator(Iterator):
 
     """
 
-    def __init__(self, data=None, minibatch_size=8):
+    def __init__(self, data=None, label=None,minibatch_size=8):
         super().__init__(data, minibatch_size)
         self.is_pair_process = False
         self.signature = None
@@ -706,7 +917,11 @@ class MetricIterator(Iterator):
             self._data = data
         elif data is not None and isinstance(data, list):
             self._data = ImageDataset(images=data)
-        self._label = ImageDataset()
+        self._label = LabelDataset()
+        if label is not None and isinstance(label, (LabelDataset)):
+            self._label= label
+        elif label is not None and isinstance(label, list):
+            self._label = LabelDataset(labels=label)
         self._unpair = ImageDataset()
         self.workers = 2
         self.itr = 0
@@ -725,6 +940,8 @@ class MetricIterator(Iterator):
 
             anchor = self.data.__getitem__(index % len(self.data)) if self.data is not None and len(self.data) > 0 else None
             positive = self.data.__getitem__(index % len(self.data)) if self.data is not None and len(self.data) > 0 else None
+            label = self.label.__getitem__(index % len(self.label)) if self.label is not None and len(self.label) > 0 else None
+
             available_list = list(range(len(self.data)))
             available_list.remove(index % len(self.data))
 
@@ -740,6 +957,8 @@ class MetricIterator(Iterator):
                     self.signature.outputs['positive'] = (-1,) + positive.shape
                 if negative is not None:
                     self.signature.outputs['negative'] = (-1,) + negative.shape
+                if label is not None:
+                    self.signature.outputs['label'] = (-1,)
 
             if anchor is not None:
                 return_data.append(anchor)
@@ -747,6 +966,8 @@ class MetricIterator(Iterator):
                 return_data.append(positive)
             if negative is not None:
                 return_data.append(negative)
+            if label is not None:
+                return_data.append(label)
 
             return tuple(return_data)
         except:
