@@ -30,7 +30,7 @@ __all__ = ['transform_func','read_image', 'read_mask', 'save_image', 'save_mask'
            'random_adjust_hue', 'random_channel_shift', 'random_cutout', 'random_rescale_crop', 'random_center_crop',
            'adjust_gamma','adjust_brightness_contrast', 'random_adjust_gamma', 'adjust_contrast', 'random_adjust_contrast', 'clahe',
            'erosion_then_dilation', 'dilation_then_erosion', 'image_erosion', 'image_dilation', 'adaptive_binarization',
-           'random_transform', 'horizontal_flip', 'random_mirror', 'to_low_resolution']
+           'random_transform', 'horizontal_flip', 'random_mirror', 'to_low_resolution','random_erasing']
 
 _session = get_session()
 _backend = _session.backend
@@ -177,34 +177,17 @@ def resize(size, keep_aspect=True, order=1, align_corner=True):
     def img_op(*image: np.ndarray):
         results = to_list(image)
         if keep_aspect:
-            h, w = image[0].shape[:2]
-            img_op.h=h
-            img_op.w=w
-            sh, sw = size
+            heigth,width  = size
+            currentHeight, currentWidth = image[0].shape[:2]
+            scale=builtins.min(heigth/currentHeight,width/currentWidth)
+            new_h=currentHeight*scale
+            new_w=currentWidth*scale
+            pad_vert = (heigth - new_h) / 2
+            pad_horz = (width - new_w) / 2
+            pad_top, pad_btm = np.floor(pad_vert).astype(int), np.ceil(pad_vert).astype(int)
+            pad_left, pad_right = np.floor(pad_horz).astype(int), np.ceil(pad_horz).astype(int)
 
-            # aspect ratio of image
-            aspect =np.true_divide(float(h),float(w))  # if on Python 2, you might need to cast as a float: float(w)/h
-            target_aspect=sw/sh
-            # compute scaling and pad sizing
-            if aspect > target_aspect:  # horizontal image
-                new_w = sw
-                new_h = np.round(new_w / aspect).astype(int)
-                pad_vert = (sh - new_h) / 2
-                pad_top, pad_btm = np.floor(pad_vert).astype(int), np.ceil(pad_vert).astype(int)
-                pad_left, pad_right = 0, 0
-            elif aspect < target_aspect:  # vertical image
-                new_h = sh
-                new_w = np.round(new_h * aspect).astype(int)
-                pad_horz = (sw - new_w) / 2
-                pad_left, pad_right = np.floor(pad_horz).astype(int), np.ceil(pad_horz).astype(int)
-                pad_top, pad_btm = 0, 0
-            else:  # square image
-                new_h, new_w = sh, sw
-                pad_left, pad_right, pad_top, pad_btm = 0, 0, 0, 0
-
-            scalex =np.true_divide(float(new_w),float(w))
-            scaley = np.true_divide(float(new_h),float(h))
-            img_op.scale = min(scalex,scaley)
+            img_op.scale = scale
             if align_corner:
                 img_op.pad_top = 0
                 img_op.pad_btm = int(pad_btm + pad_top)
@@ -222,10 +205,21 @@ def resize(size, keep_aspect=True, order=1, align_corner=True):
                 if im is None:
                     pass
                 elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox [:,4]   [:,5]
+                    class_info = None
+                    if im.shape[-1] >4:
+                        class_info = im[:, 4:]
+                        im = im[:, :4]
                     im[:, :4]=im[:, :4]*img_op.scale
+                    im[:, 0::2] +=img_op.pad_left
+                    im[:, 1::2] +=img_op.pad_top
+
+                    if class_info is not None:
+                        im=np.concatenate([im,class_info],axis=-1)
                     results[i] = im
                 elif im.ndim == 2 and im.shape[-1] in (2):  # landmark [:,2]
                     im[:, :2]=im[:, :2]*img_op.scale
+                    im[:, 0::2] += img_op.pad_left
+                    im[:, 1::2] += img_op.pad_top
                     results[i] = im
                 elif im.ndim == 2 and im.dtype == np.int64:  # bbox
                     new_im = np.zeros((size[0], size[1]))
@@ -296,7 +290,13 @@ def rescale(scale, order=1):
             if im is None or len(im) == 0:
                 pass
             elif len(im.shape) == 2 and im.shape[-1] in (4, 5, 14, 15):  # bbox
+                class_info = None
+                if im.shape[-1] >4:
+                    class_info = im[:, 4:]
+                    im = im[:, :4]
                 im[:, 0:4]=im[:, 0:4]* img_op.scale
+                if class_info is not None:
+                    im = np.concatenate([im, class_info], axis=-1)
                 results[i] = im
             elif len(im.shape) == 2 and im.shape[-1] ==2:  # landmark
                 im[:, :2]=im[:, :2]* img_op.scale
@@ -378,9 +378,7 @@ def to_rgb():
             if image.shape[0] in (1,3, 4) and image.shape[-1] >4:
                 image = image.transpose([1, 2, 0])
             if image.shape[-1] in (3, 4):
-                if image.shape[0] == 4:
-                    image = image[:, :,:3]
-                image = image[..., ::-1]
+                image = image[:,:, :3]
             elif  image.shape[-1] ==1:
                image = np.concatenate([image, image, image], -1)
             return image
@@ -489,9 +487,15 @@ def adjust_gamma(gamma=1):
 
 
 def random_adjust_gamma(gamma=(0.6, 1.4)):
-    gammamin, gammamax = gamma
-
+    gamma_range=gamma
     def img_op(image: np.ndarray):
+        gammamin, gammamax = gamma_range
+        avg_pix=image.mean()
+        if avg_pix>220:
+            gammamax=builtins.max(gammamin,1)
+        elif avg_pix<30:
+            gammamin = builtins.min(gammamax, 1)
+
         gamma = np.random.choice(np.arange(gammamin, gammamax, 0.01))
         return exposure.adjust_gamma(image, gamma)
 
@@ -590,19 +594,25 @@ def random_crop(h, w):
             if im is None:
                 pass
             elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
-
+                class_info=None
+                if im.shape[-1] ==5:
+                    class_info=im[:,4:5]
+                    im=im[:,:4]
                 im[:, 0::2] = np.clip(im[:, 0::2] - offset_x, 0, w)
                 im[:, 1::2] = np.clip(im[:, 1::2] - offset_y, 0, h)
 
                 area = (im[:, 3] - im[:, 1]) * (im[:, 2] - im[:, 0])
                 im = im[area > 0]
+
                 if len(im) > 0:
                     im[:, 0::2] += offset_x1
                     im[:, 1::2] += offset_y1
-
+                    if class_info is not None:
+                        class_info= class_info[area > 0]
+                        im=np.concatenate([im,class_info],axis=-1)
                     results[i] = im
                 else:
-                    results[i] = None
+                    results[i] = im
             elif im.ndim == 2 and im.shape[-1]==2:  # landmark
                 im[:, 0] = im[:, 0] - offset_x+offset_x1
                 im[:, 1] = im[:, 1] - offset_y+offset_y1
@@ -650,10 +660,15 @@ def random_rescale_crop(h, w, scale=(0.5, 2), order=1):
             if im is None:
                 pass
             elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
-                im[:, 0] *= scale
-                im[:, 1] *= scale
-                im[:, 2] *= scale
-                im[:, 3] *= scale
+                class_info = None
+                if im.shape[-1] >4:
+                    class_info = im[:, 4:]
+                    im = im[:, :4]
+                im=im.astype(np.float32)
+                im[:, 0] = im[:, 0] *scale
+                im[:, 1] =im[:, 1]* scale
+                im[:, 2] = im[:, 2] *scale
+                im[:, 3] =im[:, 3]* scale
 
                 im[:, 0] = np.clip(im[:, 0] - offset_x, 0, w)
                 im[:, 1] = np.clip(im[:, 1] - offset_y, 0, h)
@@ -666,6 +681,9 @@ def random_rescale_crop(h, w, scale=(0.5, 2), order=1):
                     im[:, 1] += offset_y1
                     im[:, 2] += offset_x1
                     im[:, 3] += offset_y1
+                    if class_info is not None:
+                        class_info= class_info[area > 0]
+                        im=np.concatenate([im,class_info],axis=-1)
                     results[i] = im
                 else:
                     results[i] = None
@@ -728,6 +746,10 @@ def random_center_crop(h,w, scale=(0.8, 1.2)):
             if im is None:
                 pass
             elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
+                class_info = None
+                if im.shape[-1] >4:
+                    class_info = im[:, 4:]
+                    im = im[:, :4]
                 im[:, 0] = np.clip((im[:, 0] + j) * scale + j1 - j2, 0, w)
                 im[:, 1] = np.clip((im[:, 1] + i) * scale + i1 - i2, 0, h)
                 im[:, 2] = np.clip((im[:, 2] + j) * scale + j1 - j2, 0, w)
@@ -735,6 +757,9 @@ def random_center_crop(h,w, scale=(0.8, 1.2)):
                 area = (im[:, 3] - im[:, 1]) * (im[:, 2] - im[:, 0])
                 im = im[area > 0]
                 if len(im) > 0:
+                    if class_info is not None:
+                        class_info= class_info[area > 0]
+                        im=np.concatenate([im,class_info],axis=-1)
                     results[k] = im
                 else:
                     results[k] = None
@@ -778,9 +803,14 @@ def horizontal_flip():
             if im is None:
                 pass
             elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
-                im[:, 0]=width- im[:, 0]
-                im[:, 2]=width- im[:, 2]
+                class_info = None
+                if im.shape[-1] >4:
+                    class_info = im[:, 4:]
+                    im = im[:, :4]
+                im[:, 0::2] = width - im[:, 2::-2]
                 if len(im) > 0:
+                    if class_info is not None:
+                        im=np.concatenate([im,class_info],axis=-1)
                     results[k] = im
                 else:
                     results[k] = None
@@ -803,15 +833,16 @@ def random_mirror():
             if im is None:
                 pass
             elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
-                box=im.copy()
-                print(im)
+                class_info = None
+                if im.shape[-1] >4:
+                    class_info = im[:, 4:]
+                    im = im[:, :4]
                 if img_op.rnd % 2 == 0:
-                    im[:, 0] = width - box[:, 2]
-                    im[:, 2] = width - box[:, 0]
-                    print(im)
-                    results[k] = im
-
+                    im[:, 0::2] = width - im[:, 2::-2]
 
+                if class_info is not None:
+                    im=np.concatenate([im,class_info],axis=-1)
+                results[k] = im
             elif im.ndim == 3:
                 if img_op.rnd % 2 == 0:
                     results[k] = im[:, ::-1]
@@ -847,7 +878,7 @@ def reverse_image_backend_adaption(image):
     return image
 
 
-def random_channel_shift(intensity=15):
+def random_channel_shift(intensity=0.15):
     channel_axis = -1
     inten = intensity
     def img_op(image: np.ndarray):
@@ -863,6 +894,48 @@ def random_channel_shift(intensity=15):
     return img_op
 
 
+
+def random_erasing(size_range=(0.02,0.3),transparency_range=(0.4,0.8),transparancy_ratio=0.5):
+    def img_op(image):
+        s_l ,s_h= size_range
+        r_1 = 0.3
+        r_2 = 1 / 0.3
+        h, w, c = image.shape
+        p_1 = np.random.rand()
+
+        if p_1 > 0.5:
+            return image
+
+        while True:
+            s = np.random.uniform(s_l, s_h) * h * w/4.0
+            r = np.random.uniform(r_1, r_2)
+            w1 = int(np.sqrt(s / r))
+            h1 = int(np.sqrt(s * r))
+            left = np.random.randint(0, w)
+            top = np.random.randint(0, h)
+
+            if left + w1 <= w and top + h1 <= h:
+                break
+        rr=np.random.uniform(0,1)
+        if rr<=transparancy_ratio:
+            transparancy= np.random.uniform(*transparency_range)
+            mask=np.ones_like(image)
+            mask[top:top + h1, left:left + w1, :]=0
+            image=image*(mask)+ image*(1-mask)*(transparancy)
+        else:
+
+            if rr%2==1:
+                c1 = np.random.uniform(0, 255, (h1, w1, c))
+            else:
+                c1 = np.random.uniform(0, 255)
+
+            image[top:top + h1, left:left + w1, :] = c1
+
+        return image
+
+    return img_op
+
+
 def random_cutout(img, mask):
     h, w = img.shape[:2] if _backend == 'tensorflow' or len(img.shape) == 2 else img.shape[1:3]
     cutx = random.choice(range(0, w // 4))
@@ -888,6 +961,7 @@ def random_cutout(img, mask):
     return img, mask
 
 
+
 ## denoise, smooth,
 def clahe(clip_limit=0.1, nbins=16):
     def img_op(image: np.ndarray):
@@ -1079,14 +1153,16 @@ def random_transform(rotation_range= 15, zoom_range= 0.02, shift_range= 0.02,she
     rotation = np.random.uniform(-rotation_range, rotation_range) if rotation_range!=0 else 0
     scale = np.random.uniform(1 - zoom_range, 1 + zoom_range)if zoom_range!=0 else 1
     shear= np.random.uniform( - shear_range,  shear_range)if shear_range!=0 else 0
+    shift_x = np.random.uniform(-shift_range, shift_range) if shift_range != 0 else 0
+    shift_y = np.random.uniform(-shift_range, shift_range)  if shift_range != 0 else 0
     rr = np.random.random()
     def img_op(*image: np.ndarray):
         results = to_list(image)
 
         h, w = results[0].shape[0:2]
-        tx = int(np.random.uniform(-shift_range, shift_range)* w if shift_range!=0 else 0)
-        ty = int(np.random.uniform(-shift_range, shift_range)* h if shift_range!=0 else 0)
-        mat = cv2.getRotationMatrix2D((w // 2+tx, h // 2+ty), rotation,1)
+        img_op.tx = int(shift_x* w)
+        img_op.ty = int(shift_y* h )
+        mat = cv2.getRotationMatrix2D((w // 2+img_op.tx, h // 2+img_op.ty), rotation,1)
         #mat[:, 2] += (tx, ty)
 
         cos = np.abs(mat[0, 0])
@@ -1103,7 +1179,10 @@ def random_transform(rotation_range= 15, zoom_range= 0.02, shift_range= 0.02,she
             if im is None or len(im)==0:
                 pass
             elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
-
+                class_info = None
+                if im.shape[-1] >4:
+                    class_info = im[:, 4:]
+                    im = im[:, :4]
                 # compute the new bounding dimensions of the image
                 box_w = (im[:, 2] - im[:, 0]).reshape(-1, 1)
                 box_h = (im[:, 3] - im[:, 1]).reshape(-1, 1)
@@ -1139,13 +1218,25 @@ def random_transform(rotation_range= 15, zoom_range= 0.02, shift_range= 0.02,she
 
                 if im.ndim==1:
                     im=np.expand_dims(im,0)
+
+                if rr< random_flip:
+                    im[:, 0::2] =img_op.flip_width - im[:, 2::-2]
+                im[:, 0] = np.clip(im[:,0],0,w)
+                im[:, 1] = np.clip(im[:, 1], 0, h)
+                im[:, 2] = np.clip(im[:,2], 0, w)
+                im[:, 3] = np.clip(im[:,3], 0, h)
+
                 area = (im[:, 3] - im[:, 1]) * (im[:, 2] - im[:, 0])
                 im = im[area > 0]
                 if len(im) > 0:
+                    if class_info is not None:
+                        class_info = class_info[area > 0]
+                        im=np.concatenate([im,class_info],axis=-1)
                     results[k] = im
                 else:
                     results[k] = None
             elif im.ndim == 2 and im.shape[-1] ==2:  # bbox
+                new_n=[]
                 for i in range(len(im)):
                     pts = []
                     pts.append(np.squeeze(np.array(mat_img[0]))[0] * im[i][0] + np.squeeze(np.array(mat_img[0]))[1] * im[i][1] + np.squeeze(np.array(mat_img[0]))[2])
@@ -1155,6 +1246,7 @@ def random_transform(rotation_range= 15, zoom_range= 0.02, shift_range= 0.02,she
             elif im.ndim == 3:
                 new_image = cv2.warpAffine(im.copy(), mat_img, (w, h), borderMode=cv2.BORDER_CONSTANT,borderValue=(255, 255, 255))  # , borderMode=cv2.BORDER_REPLICATE
                 if rr< random_flip:
+                    img_op.flip_width=new_image.shape[1]
                     new_image = new_image[:, ::-1]
                 results[k] = new_image
             elif im.ndim ==2:
@@ -1191,6 +1283,8 @@ def to_low_resolution(scale=2):
 
     return img_op
 
+
+
 # def image_smoothening():
 #     def img_op(image: np.ndarray):
 #         ret1, th1 = cv2.threshold(img, BINARY_THREHOLD, 255, cv2.THRESH_BINARY)
