@@ -2,6 +2,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import inspect
 import itertools
 import locale
 import os
@@ -14,7 +15,8 @@ from trident.data.image_common import check_same_size
 from trident.backend.common import OrderedDict
 from trident.backend.load_backend import get_backend
 
-__all__ = ['Sampler','SequentialSampler','RandomSampler','BatchSampler']
+__all__ = ['Sampler', 'SequentialSampler', 'RandomSampler', 'BatchSampler']
+
 
 class Sampler(object):
     r"""Base class for all Samplers.
@@ -51,6 +53,7 @@ class SequentialSampler(Sampler):
     def __len__(self):
         return len(self.data_source)
 
+
 class RandomSampler(Sampler):
     r"""Samples elements randomly. If without replacement, then sample from a shuffled dataset.
     If with replacement, then user can specify ``num_samples`` to draw.
@@ -84,7 +87,7 @@ class RandomSampler(Sampler):
     def __iter__(self):
         n = len(self.data_source)
         if self.is_bootstrap:
-            return iter(np.random.randint(high=n, low=0,size=(self.bootstrap_samples), dtype=np.int64).tolist())
+            return iter(np.random.randint(high=n, low=0, size=(self.bootstrap_samples), dtype=np.int64).tolist())
         return iter(np.random.randperm(n).tolist())
 
     def __len__(self):
@@ -107,7 +110,7 @@ class BatchSampler(Sampler):
         [[0, 1, 2], [3, 4, 5], [6, 7, 8]]
     """
 
-    def __init__(self, data_source, batch_size=1, is_shuffle=True, drop_last=False):
+    def __init__(self, data_source, batch_size=1, is_shuffle=True, drop_last=False, sample_filter=None):
         super().__init__(data_source)
         if not isinstance(batch_size, int) or isinstance(batch_size, bool) or batch_size <= 0:
             raise ValueError("batch_size should be a positive integeral value, "
@@ -118,8 +121,7 @@ class BatchSampler(Sampler):
         self.data_source = data_source
         self.batch_size = batch_size
         self.drop_last = drop_last
-        self.is_shuffle=is_shuffle
-
+        self.is_shuffle = is_shuffle
 
         idxes = np.arange(len(self.data_source))
         if len(self.data_source) % self.batch_size > 0:
@@ -129,46 +131,44 @@ class BatchSampler(Sampler):
         idxes = list(idxes)
 
         self.sampler = itertools.cycle(iter(idxes))
+        self.sample_filter = None
+        if inspect.isfunction(sample_filter) or callable(sample_filter):
+            self.sample_filter = sample_filter
 
     def __iter__(self):
-        batch =OrderedDict()
-        batch_shape = OrderedDict()
-        _data_cnt=0
+
+        batch_data = []
+
         for idx in self.sampler:
             try:
-                _return_data=self.data_source[idx]
-                if isinstance(_return_data,tuple):
-                    for i in range(len(_return_data)):
-                        if i not in batch:
-                            batch[i]=[]
-                            batch_shape[i]=[]
-                        batch[i].append(_return_data[i])
-                        batch_shape[i].append(_return_data[i].shape if isinstance(_return_data[i],np.ndarray) else (-1))
-                else:
-                    if  0not in batch:
-                        batch[0] = []
-                        batch_shape[0] = []
-                    batch[0].append(_return_data)
-                    batch_shape[0].append(_return_data.shape)
-                _data_cnt+=1
+                _return_data = self.data_source[idx]
+                # filter sample
+                if self.sample_filter is None or self.sample_filter(_return_data):
+                    if not isinstance(_return_data, tuple):
+                        _return_data = _return_data,
+                    batch_data.append(_return_data)
+
             except Exception as e:
                 print(e)
 
-            if _data_cnt== self.batch_size:
-                returnData=[ np.asarray(batch.value_list[i]) if len(list(set(batch_shape.value_list[i])))==1 else batch.value_list[i]  for i in range(len(batch.value_list)) ]
+            if len(batch_data) == self.batch_size:
+                returnData = list(zip(*batch_data))
+                for i in range(len(returnData)):
+                    returnData[i] = np.asarray(list(returnData[i]))
+
                 yield tuple(returnData)
-                batch = OrderedDict()
-                batch_shape = OrderedDict()
-                _data_cnt = 0
-        if len(batch) == 0:
-            self.reset()
-            raise StopIteration
+                batch_data = []
+
+
+
+        self.reset()
+        # raise StopIteration
 
     def __len__(self):
         if self.drop_last:
             return len(self.data_source) // self.batch_size
         else:
-            return (len(self.data_source)+ self.batch_size - 1) // self.batch_size
+            return (len(self.data_source) + self.batch_size - 1) // self.batch_size
 
     def reset(self):
         idxes = np.arange(len(self.data_source))
