@@ -37,6 +37,7 @@ def group_std(x, groups, eps = 1e-5):
 
 class BatchNorm(Layer):
     """Applies Batch Normalization over a 4D input (a mini-batch of 2D inputs
+
     with additional channel dimension) as described in the paper
     `Batch Normalization: Accelerating Deep Network Training by Reducing Internal Covariate Shift`_ .
 
@@ -78,8 +79,8 @@ class BatchNorm(Layer):
         https://arxiv.org/abs/1502.03167
 
     """
-
-    def __init__(self,  momentum=0.1, affine=True, track_running_stats=True, eps=1e-5,name=None, **kwargs):
+    _version = 2
+    def __init__(self,  momentum=0.1, affine=True, track_running_stats=True, eps=1e-5,in_sequence=False,name=None, **kwargs):
         """
         Args:
         eps: a value added to the denominator for numerical stability.
@@ -102,27 +103,40 @@ class BatchNorm(Layer):
 
         """
 
-        super().__init__(name=name)
-
+        super().__init__(in_sequence=in_sequence,name=name)
         self.eps = eps
         self.momentum = momentum
         self.affine = affine
         self.track_running_stats = track_running_stats
 
+
     def reset_running_stats(self):
         if self.track_running_stats:
             self.running_mean.zero_()
             self.running_var.fill_(1)
             self.num_batches_tracked.zero_()
-        if self.affine :
-            init.ones_(self.weight)
-            init.zeros_(self.bias)
+
+
     def reset_parameters(self):
         self.reset_running_stats()
         if self.affine:
             init.ones_(self.weight)
             init.zeros_(self.bias)
 
+    def _load_from_state_dict(self, state_dict, prefix, local_metadata, strict,
+                              missing_keys, unexpected_keys, error_msgs):
+        version = local_metadata.get('version', None)
+
+        if (version is None or version < 2) and self.track_running_stats:
+            # at version 2: added num_batches_tracked buffer
+            #               this should have a default value of 0
+            num_batches_tracked_key = prefix + 'num_batches_tracked'
+            if num_batches_tracked_key not in state_dict:
+                state_dict[num_batches_tracked_key] = torch.tensor(0, dtype=torch.long)
+
+        super(BatchNorm, self)._load_from_state_dict(
+            state_dict, prefix, local_metadata, strict,
+            missing_keys, unexpected_keys, error_msgs)
 
     def build(self, input_shape):
         if self._built == False:
@@ -143,12 +157,19 @@ class BatchNorm(Layer):
                 self.register_parameter('running_mean', None)
                 self.register_parameter('running_var', None)
                 self.register_parameter('num_batches_tracked', None)
+
             self.reset_parameters()
             self.to(get_device())
             self._built = True
+
     def forward(self, *x):
         x = enforce_singleton(x)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x=x.permute(0, 2, 1)
 
+        # exponential_average_factor is set to self.momentum
+        # (when it is available) only so that it gets updated
+        # in ONNX graph when this node is exported to ONNX.
         if self.momentum is None:
             exponential_average_factor = 0.0
         else:
@@ -157,18 +178,42 @@ class BatchNorm(Layer):
         if self.training and self.track_running_stats:
             # TODO: if statement only here to tell the jit to skip emitting this when it is None
             if self.num_batches_tracked is not None:
-                self.num_batches_tracked += 1
+                self.num_batches_tracked = self.num_batches_tracked + 1
                 if self.momentum is None:  # use cumulative moving average
                     exponential_average_factor = 1.0 / float(self.num_batches_tracked)
                 else:  # use exponential moving average
                     exponential_average_factor = self.momentum
-        return F.batch_norm(x, self.running_mean, self.running_var, self.weight, self.bias, self.training or not self.track_running_stats, exponential_average_factor, self.eps)
+
+        """ Decide whether the mini-batch stats should be used for normalization rather than the buffers.
+                Mini-batch stats are used in training mode, and in eval mode when buffers are None.
+                """
+        if self.training:
+            bn_training = True
+        else:
+            bn_training = (self.running_mean is None) and (self.running_var is None)
+
+        """Buffers are only updated if they are to be tracked and we are in training mode. Thus they only need to be
+                passed when the update should occur (i.e. in training mode when they are tracked), or when buffer stats are
+                used for normalization (i.e. in eval mode when buffers are not None).
+                """
+        x= F.batch_norm(
+            x,
+            # If buffers are not to be tracked, ensure that they won't be updated
+            self.running_mean if not self.training or self.track_running_stats else None,
+            self.running_var if not self.training or self.track_running_stats else None,
+            self.weight, self.bias, bn_training, exponential_average_factor, self.eps)
+
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x=x.permute(0, 2, 1)
+        return x
+
     def extra_repr(self):
         return '{input_filters}, eps={eps}, momentum={momentum}, affine={affine}, ' \
                'track_running_stats={track_running_stats}'.format(**self.__dict__)
 
     def _load_from_state_dict(self, state_dict, prefix, local_metadata, strict,
                               missing_keys, unexpected_keys, error_msgs):
+
         version = local_metadata.get('version', None)
 
         if (version is None or version < 2) and self.track_running_stats:
@@ -181,6 +226,7 @@ class BatchNorm(Layer):
         super(BatchNorm, self)._load_from_state_dict(
             state_dict, prefix, local_metadata, strict,
             missing_keys, unexpected_keys, error_msgs)
+        self.eval()
 BatchNorm1d=BatchNorm
 BatchNorm2d=BatchNorm
 BatchNorm3d=BatchNorm
@@ -212,7 +258,7 @@ class GroupNorm(Layer):
 
     """
 
-    def __init__(self, num_groups=16,affine=True, eps=1e-5,name=None, **kwargs):
+    def __init__(self, num_groups=16,affine=True, eps=1e-5,in_sequence=False,name=None, **kwargs):
         """
         Args:
             num_groups (int): number of groups to separate the channels into
@@ -228,7 +274,7 @@ class GroupNorm(Layer):
             (2, 64, 128, 128)
 
         """
-        super().__init__(name=name)
+        super().__init__(in_sequence=in_sequence,name=name)
         self.affine=affine
         self.num_groups = num_groups
         self.eps = eps
@@ -248,7 +294,13 @@ class GroupNorm(Layer):
                 self._built = True
     def forward(self, *x):
         x = enforce_singleton(x)
-        return F.group_norm(x, self.num_groups, self.weight, self.bias, self.eps)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
+        x= F.group_norm(x, self.num_groups, self.weight, self.bias, self.eps)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x=x.permute(0, 2, 1)
+        return x
+
 GroupNorm2d=GroupNorm
 GroupNorm3d=GroupNorm
 
@@ -300,7 +352,7 @@ class InstanceNorm(Layer):
         https://arxiv.org/abs/1607.08022
     """
 
-    def __init__(self,momentum=0.1, affine=True, track_running_stats=True, eps=1e-5,axis=1,name=None, **kwargs):
+    def __init__(self,momentum=0.1, affine=True, track_running_stats=True, eps=1e-5,in_sequence=False,axis=1,name=None, **kwargs):
         """
         Args:
             num_features: :math:`C` from an expected input of size
@@ -322,7 +374,7 @@ class InstanceNorm(Layer):
             (2, 64, 128, 128)
 
         """
-        super().__init__(name=name)
+        super().__init__(in_sequence=in_sequence,name=name)
         self.eps = _epsilon
         self.momentum = momentum
         self.affine = affine
@@ -361,9 +413,15 @@ class InstanceNorm(Layer):
             self._built = True
     def forward(self, *x):
         x = enforce_singleton(x)
-        return F.instance_norm(x, self.running_mean, self.running_var, self.weight, self.bias,
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
+        x= F.instance_norm(x, self.running_mean, self.running_var, self.weight, self.bias,
             self.training or not self.track_running_stats, self.momentum, self.eps)
 
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
+        return x
+
 InstanceNorm2d=InstanceNorm
 InstanceNorm3d=InstanceNorm
 
@@ -400,7 +458,7 @@ class LayerNorm(Layer):
     .. _`Layer Normalization`: https://arxiv.org/abs/1607.06450
 
     """
-    def __init__(self, normalized_shape, eps=1e-5, elementwise_affine=True,name=None, **kwargs):
+    def __init__(self, normalized_shape, eps=1e-5, elementwise_affine=True,in_sequence=False,name=None, **kwargs):
         """
     Args:
         normalized_shape (int or list or torch.Size): input shape from an expected input
@@ -432,22 +490,28 @@ class LayerNorm(Layer):
         >>> output = m(input)
 
         """
-        super().__init__(name=name)
+        super().__init__(in_sequence=in_sequence,name=name)
         if isinstance(normalized_shape, numbers.Integral):
             normalized_shape = (normalized_shape,)
         self.normalized_shape = tuple(normalized_shape)
         self.eps = eps
         self.elementwise_affine = elementwise_affine
 
+
+
     def build(self, input_shape):
         if self._built == False :
-            if self.affine:
-                self.weight = Parameter(ones((self.normalized_shape)))
-                self.bias = Parameter(zeros((self.normalized_shape)))
+            self.register_parameter('weight',Parameter(ones((self.normalized_shape))))
+            self.register_parameter('bias', Parameter(zeros((self.normalized_shape))))
             self._built=True
     def forward(self, *x):
         x = enforce_singleton(x)
-        return F.layer_norm(x, self.normalized_shape, self.weight, self.bias, self.eps)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
+        x= F.layer_norm(x, self.normalized_shape, self.weight, self.bias, self.eps)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x=x.permute(0, 2, 1)
+        return x
         # mean = x.mean(dim=self.axis, keepdim=True).detach()
         # std = x.std(dim=self.axis, keepdim=True).detach()
         # return self.weight * (x - mean) / (std + self._eps) +self.bias
@@ -457,25 +521,37 @@ LayerNorm3d=LayerNorm
 
 
 class L2Norm(Layer):
-    def __init__(self, name=None, **kwargs):
-        super().__init__(name=name)
+    def __init__(self,in_sequence=False, axis=1,name=None, **kwargs):
+        super().__init__(in_sequence=in_sequence,name=name)
         self.eps=epsilon()
+        self.axis=axis
 
     def build(self, input_shape):
         if self._built == False :
             self._built = True
     def forward(self, *x):
         x = enforce_singleton(x)
-        return l2_normalize(x)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
+        x= l2_normalize(x,axis=self.axis,keepdims=True)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
+        return x
 
 
 
 class PixelNorm(Layer):
-    def __init__(self,name=None, **kwargs):
-        super(PixelNorm, self).__init__(name=name)
+    def __init__(self,in_sequence=False,name=None, **kwargs):
+        super(PixelNorm, self).__init__(in_sequence=in_sequence,name=name)
 
     def forward(self, x):
-        return x / torch.sqrt(torch.mean(x ** 2, dim=1, keepdim=True) + 1e-8)
+        x = enforce_singleton(x)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
+        x= x / torch.sqrt(torch.mean(x ** 2, dim=1, keepdim=True) + 1e-8)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x=x.permute(0, 2, 1)
+        return x
 
 
 class SpectralNorm(Layer):
@@ -520,8 +596,8 @@ class SpectralNorm(Layer):
         torch.Size([40])
 
     """
-    def __init__(self, module, name='weight', power_iterations=1):
-        super(SpectralNorm, self).__init__()
+    def __init__(self, module, name='weight', power_iterations=1,in_sequence=False,**kwargs):
+        super(SpectralNorm, self).__init__(in_sequence=in_sequence,name=name)
         self.module = module
         self.name = name
         self.power_iterations = power_iterations
@@ -574,9 +650,14 @@ class SpectralNorm(Layer):
                 self._make_params()
             self._built = True
     def forward(self, *x):
-        x=enforce_singleton(x)
+        x = enforce_singleton(x)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x = x.permute(0, 2, 1)
         self._update_u_v()
-        return self.module(x)
+        x= self.module(x)
+        if hasattr(self,'in_sequence') and self.in_sequence:
+            x=x.permute(0, 2, 1)
+        return x
 
 
 class EvoNormB0(Layer):
