@@ -565,27 +565,44 @@ def get_activation(fn_name):
 
 
     """
-    fn_modules = ['trident.backend.tensorflow_ops','trident.layers.tensorflow_activations']
+    fn_modules = ['trident.backend.tensorflow_ops','trident.layers.tensorflow_activations','tensorflow.python.framework.ops.nn_ops']
 
     if fn_name is None:
         return None
-    elif inspect.isclass(fn_name) and inspect._is_type(fn_name):
-        return fn_name()
-    elif isinstance(fn_name, Layer):
-        return fn_name
-    elif isinstance(fn_name, str) :
-        pooling_class = get_function(fn_name, fn_modules)
-        return pooling_class
-    elif isinstance(fn_name, str) and fn_name in __all__:
-        pooling_class = get_class(fn_name, fn_modules)
-        return pooling_class()
-    elif isinstance(fn_name, str) and snake2camel(fn_name) in __all__:
-        pooling_class = get_class(snake2camel(fn_name), fn_modules)
-        return pooling_class()
-    elif inspect.isfunction(fn_name) or callable(fn_name):
-        result = inspect.getfullargspec(fn_name)
-        if 1 == len(result.args):
+    try:
+        if isinstance(fn_name, str):
+            if camel2snake(fn_name)== fn_name or fn_name.lower()== fn_name:
+                if fn_name == 'p_relu' or fn_name == 'prelu':
+                    return PRelu()
+                activation_fn = get_function(fn_name, ['trident.backend.tensorflow_ops',
+                                                       'trident.layers.tensorflow_activations'] if fn_name in __all__ else fn_modules)
+                return activation_fn
+            else:
+                try:
+                    activation_fn = get_class(snake2camel(fn_name), fn_modules)
+                    return activation_fn()
+                except Exception:
+                    activation_fn = get_class(fn_name, ['trident.backend.tensorflow_ops',
+                                                        'trident.layers.tensorflow_activations'] if fn_name in __all__ else fn_modules)
+                    return activation_fn()
+        elif getattr(fn_name, '__module__', None) == 'trident.layers.pytorch_activations':
+            if inspect.isfunction(fn_name):
+                return fn_name
+            elif inspect.isclass(fn_name) and inspect._is_type(fn_name):
+                return fn_name()
+            elif isinstance(fn_name, Layer):
+                return fn_name
+        elif inspect.isfunction(fn_name) and getattr(fn_name, '__module__', None) == 'trident.backend.tensorflow_ops':
             return fn_name
-    else:
-        raise ValueError('Unknown pooling function/ class')
+
+        else:
+            if callable(fn_name):
+                result = inspect.getfullargspec(fn_name)
+                if 1 <= len(result.args) <= 2:
+                    return fn_name if inspect.isfunction(fn_name) else fn_name()
+                else:
+                    raise ValueError('Unknown activation function/ class')
+    except Exception as e:
+        print(e)
+        return None
 
