@@ -112,10 +112,11 @@ class BatchNorm(Layer):
         if isinstance(axis, (list, tuple)):
             self.axis = axis[:]
         elif isinstance(axis, int):
-            self.axis = axis
+            self.axis = [axis]
         else:
             raise TypeError('Expected an int or a list/tuple of ints for the '
                             'argument \'axis\', but received: %r' % axis)
+
         self.eps = eps
         self.momentum = momentum
         self.affine = affine
@@ -152,8 +153,7 @@ class BatchNorm(Layer):
             ndims = len(input_shape.dims)
 
             # Convert axis to list and resolve negatives
-            if isinstance(self.axis, int):
-                self.axis = [self.axis]
+
 
 
             if self.affine:
