@@ -21,16 +21,16 @@ from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
 
-from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
-from ..data.image_common import *
-from ..data.utils import download_file_from_google_drive
-from ..layers.pytorch_activations import get_activation, Identity, LeakyRelu
-from ..layers.pytorch_blocks import *
-from ..layers.pytorch_layers import *
-from ..layers.pytorch_normalizations import get_normalization, BatchNorm
-from ..layers.pytorch_pooling import *
-from ..optims.pytorch_trainer import *
+from trident.backend.common import *
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
+from trident.data.image_common import *
+from trident.data.utils import download_file_from_google_drive
+from trident.layers.pytorch_activations import get_activation, Identity, LeakyRelu
+from trident.layers.pytorch_blocks import *
+from trident.layers.pytorch_layers import *
+from trident.layers.pytorch_normalizations import get_normalization, BatchNorm
+from trident.layers.pytorch_pooling import *
+from trident.optims.pytorch_trainer import *
 
 __all__ = ['gan_builder', 'UpsampleMode', 'BuildBlockMode']
 
