@@ -74,10 +74,10 @@ def inverted_residual(in_filters,num_filters=64,strides=1,expansion = 4,name='')
     mid_filters= int(round(in_filters * expansion))
     layers=[]
     if expansion!=1 :
-        layers.append(Conv2d_Block((1,1),num_filters=mid_filters,strides=1,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu6',name=name + '_{0}_conv'.format(len(layers))))
+        layers.append(Conv2d_Block((1,1),num_filters=mid_filters,strides=1,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu6'))
 
-    layers.append(DepthwiseConv2d_Block((3, 3), depth_multiplier=1, strides=strides, auto_pad=True,padding_mode='zero', normalization='batch', activation='relu6', name=name + '_{0}_conv'.format(len(layers))))
-    layers.append(Conv2d_Block((1, 1), num_filters=num_filters, strides=1, auto_pad=False, padding_mode='zero', normalization='batch', activation=None, name=name + '_{0}_conv'.format(len(layers))))
+    layers.append(DepthwiseConv2d_Block((3, 3), depth_multiplier=1, strides=strides, auto_pad=True,padding_mode='zero', normalization='batch', activation='relu6'))
+    layers.append(Conv2d_Block((1, 1), num_filters=num_filters, strides=1, auto_pad=False, padding_mode='zero', normalization='batch', activation=None))
     if  strides == 1 and in_filters==num_filters:
         return ShortCut2d(Sequential(*layers), Identity(), activation=None)
     else:
@@ -100,7 +100,7 @@ def MobileNet( input_shape=(3, 224, 224), classes=1000, use_bias=False, width_mu
         output_filters = _make_divisible(c * width_mult, round_nearest)
         for i in range(n):
             strides = s if i == 0 else 1
-            features.append( inverted_residual(input_filters,num_filters=output_filters, strides=strides, expansion=t,name='irb_{0}'.format(i)))
+            features.append( inverted_residual(input_filters,num_filters=output_filters, strides=strides, expansion=t))
             input_filters = output_filters
     features.append(Conv2d_Block((1,1), last_filters,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu6'))
     mobilenet.add_module('features',Sequential(*features,name='features'))
