@@ -49,6 +49,7 @@ if not os.path.exists(dirname):
         pass
 
 model_urls = {
+    'resnet18': '156C4a0_nts8QbjCE8YWbA-QbvCnTrfb5',
     'resnet50': '1dYlgpFtqi87KDG54_db4ALWKLARxCWMS',
     'resnet101': '17moUOsGynsWALLHyv3yprHWbbDMrdiOP',
     'resnet152': '1BIaHb7_qunUVvt4TDAwonSKI2jYg4Ybj',
@@ -116,12 +117,11 @@ def ResNet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bias=
         conv_shortcut=False
         if strides!=1 or block is bottleneck:
             conv_shortcut=True
-
         layers = []
-        layers.append(block(num_filters=num_filters, strides=strides, expansion = 4, conv_shortcut=conv_shortcut,use_bias=use_bias, name=layer_name+'.0'))
+        layers.append(block(num_filters=num_filters, strides=strides, expansion = 4, conv_shortcut=conv_shortcut,use_bias=use_bias, name=layer_name+'_0'))
 
         for k in range(1, blocklayers):
-            layers.append(block(num_filters=num_filters,  strides=1, expansion = 4, conv_shortcut=False, use_bias=use_bias,name=layer_name+'.{0}'.format(k)))
+            layers.append(block(num_filters=num_filters,  strides=1, expansion = 4, conv_shortcut=False, use_bias=use_bias,name=layer_name+'_{0}'.format(k)))
 
         laters_block=Sequential(*layers)
         laters_block.name=layer_name
@@ -175,7 +175,6 @@ def ResNet18(include_top=True,
             if classes != 1000:
                 resnet18.class_names = []
                 recovery_model.fc = Dense(classes, activation=None, name='fc')
-                recovery_model.fc.input_shape = recovery_model.avg_pool.output_shape
         resnet18.model = recovery_model
     return resnet18
 
@@ -204,7 +203,6 @@ def ResNet50(include_top=True,
             if classes!=1000:
                 resnet50.class_names = []
                 recovery_model.fc= Dense(classes, activation=None, name='fc')
-                recovery_model.fc.input_shape=recovery_model.avg_pool.output_shape
 
         resnet50.model=recovery_model
     return resnet50
@@ -234,8 +232,6 @@ def ResNet101(include_top=True,
             if classes != 1000:
                 resnet101.class_names = []
                 recovery_model.fc = Dense(classes, activation=None, name='fc')
-                recovery_model.fc.input_shape=recovery_model.avg_pool.output_shape
-
         resnet101.model=recovery_model
     return resnet101
 
@@ -265,8 +261,6 @@ def ResNet152(include_top=True,
             if classes != 1000:
                 resnet152.class_names=[]
                 recovery_model.fc = Dense(classes, activation=None, name='fc')
-                recovery_model.fc.input_shape=recovery_model.avg_pool.output_shape
-
         resnet152.model=recovery_model
     return resnet152
 
