@@ -49,7 +49,7 @@ if not os.path.exists(dirname):
         pass
 
 model_urls = {
-    'se_resnet50': '1dYlgpFtqi87KDG54_db4ALWKLARxCWMS',
+    'se_resnet50': '1vq0uueiHXuHSEFhb02GoEuPzLwrDW_Mb',
     'se_resnet101': '17moUOsGynsWALLHyv3yprHWbbDMrdiOP',
     'se_resnet152': '1BIaHb7_qunUVvt4TDAwonSKI2jYg4Ybj',
 }
@@ -231,7 +231,7 @@ def SE_ResNet152(include_top=True,
         input_shape=(3, 224, 224)
     resnet152 =SE_ResNet(se_bottleneck [3, 8, 36, 3], input_shape,num_classes=classes,include_top=include_top, model_name='resnet152')
     if pretrained==True:
-        download_model_from_google_drive(model_urls['resnet152'],dirname,'se_resnet152.pth')
+        download_model_from_google_drive(model_urls['se_resnet50'],dirname,'se_resnet152.pth')
         recovery_model=torch.load(os.path.join(dirname,'se_resnet152.pth'))
         recovery_model.name = 'se_resnet152'
         recovery_model.eval()
