@@ -3,6 +3,7 @@ from __future__ import division
 from __future__ import print_function
 
 import copy
+import gc
 import inspect
 import os
 import random
@@ -83,7 +84,7 @@ class Model(ModelBase):
         super(Model, self).__init__(inputs, input_shape,output)
 
 
-    def _initial_graph(self, inputs=None, output=None, input_shape=None,initializer=None):
+    def _initial_graph(self, inputs=None, input_shape=None,output=None,initializer=None):
         if output is None:
             raise ValueError('There is at least one output')
         if isinstance(output,(np.ndarray,torch.Tensor)) and input_shape is None:
@@ -106,13 +107,16 @@ class Model(ModelBase):
                     inp=inputs[m]
                     if is_tensor(inp) or isinstance(inp,np.ndarray):
                         input_name = 'input_{0}'.format(m)
-                        self.inputs[input_name] = list(int_shape(inp))[1:]
+                        self.inputs[input_name] = list(int_shape(inp))[self.batch_index+1:]
         elif isinstance(inputs, dict):
             for k, v in inputs.items():
                 if is_tensor(v)or isinstance(v,np.ndarray):
-                    self.inputs[k] =  list(int_shape(v))[1:]
+                    self.inputs[k] =  list(int_shape(v))[self.batch_index+1:]
         elif is_tensor(inputs):
-            self.inputs['input'] = list(int_shape(inputs))[1:]
+            self.inputs['input'] = list(int_shape(inputs))[self.batch_index+1:]
+        elif isinstance(inputs,np.ndarray):
+            inputs=to_tensor(inputs)
+            self.inputs['input'] = list(int_shape(inputs))[self.batch_index+1:]
 
         #single model
         if isinstance(output, (Layer, nn.Module)):
@@ -153,17 +157,17 @@ class Model(ModelBase):
                     out = output(dummay_input)
                 self._model = output
                 if isinstance(out, torch.Tensor):
-                    self._outputs['output'] = to_list(out.size())[1:]
-                    self._targets['target'] = to_list(out.size())[1:]
+                    self._outputs['output'] = to_list(out.size())[self.batch_index+1:]
+                    self._targets['target'] = to_list(out.size())[self.batch_index+1:]
                 else:
                     for i in range(len(out)):
-                        self._outputs['output_{0}'.format(i)] = to_list(out[i].size())[1:]
-                        self._targets['target_{0}'.format(i)] = to_list(out[i].size())[1:]
+                        self._outputs['output_{0}'.format(i)] = to_list(out[i].size())[self.batch_index+1:]
+                        self._targets['target_{0}'.format(i)] = to_list(out[i].size())[self.batch_index+1:]
             self._model.signature = get_signature(self._model.forward,'model')
             self._model.signature.inputs = copy.deepcopy(self.inputs)
             self._model.signature.outputs=copy.deepcopy(self._outputs)
             self._signature=self._model.signature
-        elif isinstance(output, (List[nn.Module],Tuple[nn.Module])):
+        elif isinstance(output, (list,tuple)) and  all([isinstance(m,(nn.Module)) for m in output]):
             output_list = []
             model_list = []
             dummay_input = to_tensor(np.random.standard_normal((1,) + tuple(input_shape)).astype(np.float32)).to(get_device()) if not is_tensor(inputs) else inputs.to(get_device())
@@ -180,8 +184,8 @@ class Model(ModelBase):
             self._model = model
             self.name = model.name
             for i in range(len(output_list)):
-                self._outputs['output_{0}'.format(i)] = list(int_shape(output_list[i]))[1:]
-                self._targets['target_{0}'.format(i)] = list(int_shape(output_list[i]))[1:]
+                self._outputs['output_{0}'.format(i)] = list(int_shape(output_list[i]))[self.batch_index+1:]
+                self._targets['target_{0}'.format(i)] = list(int_shape(output_list[i]))[self.batch_index+1:]
             self._model.signature = get_signature(self._model.forward,'model')
             self._model.signature.inputs = copy.deepcopy(self.inputs)
             self._model.signature.outputs = copy.deepcopy(self._outputs)
@@ -189,10 +193,10 @@ class Model(ModelBase):
         elif isinstance(output,(np.ndarray,torch.Tensor)):
             #style transfer , or adversarial attack
             self._model =to_tensor(output,requires_grad=True)
-            self._outputs['output'] = list(int_shape(self._model))[1:]
-            self._targets['target'] = list(int_shape(self._model))[1:]
-            self._model.signature =Signature('model')
-            self._model.signature.inputs['x']=list(int_shape(self._model))[1:]
+            out=self._model
+            self._outputs['output'] = to_list(out.size())[self.batch_index + 1:]
+            self._targets['target'] = to_list(out.size())[self.batch_index + 1:]
+            self._model.signature =Signature()
             self._model.signature.outputs = copy.deepcopy(self._outputs)
             self._signature = self._model.signature
         else:
@@ -300,6 +304,7 @@ class Model(ModelBase):
                 argnames = get_signature(self._losses[alias].forward,alias)
             else:
                 argnames = get_signature(self._losses[alias].__call__,alias)
+
         elif not inspect.isfunction(loss) and callable(loss):
             alias = loss.__class__.__name__ if alias is  None or len(alias) == 0 else alias
             if alias in self._losses:
@@ -341,46 +346,7 @@ class Model(ModelBase):
             except:
                 print(argnames)
 
-
-        # if isinstance(self._losses[alias], Layer) and len(list(self._losses[alias].parameters()))>0:
-        #     para_dict={'params': list(self._losses[alias].parameters())}
-        #     for k,v in self.optimizer.param_groups[0].items():
-        #         if k!='params':
-        #             para_dict[k]=v
-        #
-        #     self.optimizer.param_groups.append(para_dict)
-        #     print('add loss parameters to optimizer ')
-
-
         self.loss_weights[alias] = float(loss_weight)
-
-        # outputs = self.outputs
-        # targets = self.targets
-        # if all([k  in targets.key_list or k  in outputs.key_list for k  in argnames.inputs.key_list]):
-        #     pass
-        # elif outputs is not None and len(outputs) == 1 and len(argnames) == 2 and argnames.inputs.key_list[0].lower() in ['input','output','y_pred','y','pred','result'] and  argnames.key_list[1] in ['target','label','y_true']:
-        #     argnames = OrderedDict()
-        #     argnames[outputs.key_list[0]] = outputs[outputs.key_list[0]]
-        #     argnames[targets.key_list[0]] = targets[targets.key_list[0]]
-        # elif outputs is not None and len(outputs) == 1 and len(argnames) == 2 :
-        #     argnames[argnames.key_list[0]] = outputs[outputs.key_list[0]]
-        #     argnames[argnames.key_list[1]] = targets[targets.key_list[0]]
-        # elif outputs is not None and len(outputs) > 1:
-        #     output_idx = list(output_idx) if isinstance(output_idx, (list, tuple)) else [output_idx]
-        #     if len(output_idx) == 1 and len(argnames) == 2:
-        #         argnames = OrderedDict()
-        #         out = outputs.key_list[output_idx[0]]
-        #         target = targets.key_list[output_idx[0]]
-        #         argnames[argnames.key_list[0]] = outputs[out]
-        #         argnames[argnames.key_list[1]] = targets[target]
-        #     elif len(output_idx) > 1 and len(argnames) == 2 * len(output_idx):
-        #         for idx in output_idx:
-        #             out = outputs.key_list[idx]
-        #             target = targets.key_list[idx]
-        #             if out in argnames:
-        #                 argnames[out] = outputs[out]
-        #             if target in argnames:
-        #                 argnames[target] = targets[target]
         self._losses[alias].__name__ = alias
         #self._losses[alias].signature = argnames
         self._losses[alias].start_epoch = start_epoch
@@ -459,7 +425,7 @@ class Model(ModelBase):
         reg_fn = None
         if isinstance(reg, str):
             reg_fn = get_reg(reg)
-        elif reg is callable:
+        elif reg is callable or inspect.isfunction(reg):
             reg_fn = reg
         args = get_signature(reg_fn)
         if 'reg_weight' in args:
@@ -584,7 +550,9 @@ class Model(ModelBase):
         #     self.optimizer.param_groups[0]['lr'] = self.base_lr
         #     self.training_context['current_lr'] =self.base_lr
         if self.model.device == 'cuda':
+            torch.cuda.synchronize()
             torch.cuda.empty_cache()
+            gc.collect()
 
     def do_on_epoch_end(self):
         pass  # if self.training_context['current_epoch'] > self.warmup:  #     if self.lr_scheduler is not None:  #         self.lr_scheduler.step(np.array(self.training_context['metrics'][list(self._metrics.keys())[0]]).mean())  #         self.training_context['current_lr'] = self.optimizer.lr  #     if self.optimizer.param_groups[0]['lr'] < 1e-8:  #         self.optimizer.param_groups[0]['lr'] = 0.05 * self.base_lr  #         self.training_context['current_lr'] =  0.05 * self.base_lr  # elif  self.warmup>0 and self.training_context['current_epoch'] == self.warmup:  #     self.optimizer.adjust_learning_rate(self.base_lr, True)  #     self.training_context['current_lr'] =self.base_lr  # elif   self.warmup>0 and self.training_context['current_epoch'] < self.warmup:  #     self.optimizer.adjust_learning_rate(1e-5*(self.training_context['current_epoch']+1), True)  #     self.training_context['current_lr'] = 1e-5*(self.training_context['current_epoch']+1)
@@ -593,91 +561,105 @@ class Model(ModelBase):
         if self.model.device == 'cuda':
             torch.cuda.empty_cache()
 
+
     def do_on_batch_end(self):
-        if self.training_context['current_batch'] == 0:
+        if self.training_context['current_batch'] == 0 and self.training_context['is_print_batch_progress']==True:
             temp = OrderedDict()
             for k in self.training_context['losses'].key_list:
                 if len(self.training_context['losses'][k])>0:
                     temp[k] = self.training_context['losses'][k][-1]
             print(temp)
+            if is_tensor(self._model):
+                if self._model.grad is not None:
+                    self._model.requires_grad=False
+                    self._model.requires_grad=True
+            # elif isinstance(self._model, nn.Module):
+            #     self._model.zero_grad()
 
     def do_on_data_received(self, train_data, test_data):
 
         # fields=train_data._fields
         # for i in range(len(fields)):
 
-        if 'data_feed' not in self.training_context or len(self.training_context['data_feed']) == 0:
+        if 'data_feed' not in self.training_context or len(self.training_context['data_feed']) == 0 or self.training_context['current_batch']+self.training_context['current_epoch']==0:
             try:
-                data_feed = OrderedDict()
+
+                data_feed = OrderedDict() if 'data_feed' not in self.training_context else self.training_context['data_feed']
                 inshapes = self.inputs.value_list
                 outshapes = self.targets.value_list
                 available_fields = copy.deepcopy(train_data.key_list)
                 if train_data is not None:
                     # check input
                     for arg in self._model.signature.inputs.key_list:
-                        data_feed[arg] = ''
-                        if len(train_data) == 1 and len(self._model.signature.inputs.key_list)==1:
-                            data_feed[arg] = train_data.key_list[0]
-                            available_fields.remove(train_data.key_list[0])
-                        elif arg in available_fields:
-                            data_feed[arg] = arg
+                        if  arg in data_feed and arg in available_fields:
                             available_fields.remove(arg)
-                        elif arg in ['x', 'input'] and 'data' in available_fields:
-                            data_feed[arg] = 'data'
-                            available_fields.remove('data')
-                        elif arg in ['x', 'input'] and 'image' in available_fields:
-                            data_feed[arg] = 'image'
-                            available_fields.remove('image')
-                        elif arg == 'x' and 'input' in available_fields:
-                            data_feed[arg] = 'input'
-                            available_fields.remove('input')
-                        elif len(self._model.signature.inputs.key_list) == 1:
-                            for item in available_fields:
-                                data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
-                                if 'target' not in item and 'output' != item and data_shape == inshapes[0]:
-                                    data_feed[arg] = item
-                                    available_fields.remove(item)
-                                    break
                         else:
-                            Warning(
-                                'input argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
-                                    arg))
+                            data_feed[arg] = ''
+                            if len(train_data) == 1 and len(self._model.signature.inputs.key_list)==1:
+                                data_feed[arg] = train_data.key_list[0]
+                                available_fields.remove(train_data.key_list[0])
+                            elif arg in available_fields:
+                                data_feed[arg] = arg
+                                available_fields.remove(arg)
+                            elif arg in ['x', 'input'] and 'data' in available_fields:
+                                data_feed[arg] = 'data'
+                                available_fields.remove('data')
+                            elif arg in ['x', 'input'] and 'image' in available_fields:
+                                data_feed[arg] = 'image'
+                                available_fields.remove('image')
+                            elif arg == 'x' and 'input' in available_fields:
+                                data_feed[arg] = 'input'
+                                available_fields.remove('input')
+                            elif len(self._model.signature.inputs.key_list) == 1:
+                                for item in available_fields:
+                                    data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
+                                    if 'target' not in item and 'output' != item and data_shape == inshapes[0]:
+                                        data_feed[arg] = item
+                                        available_fields.remove(item)
+                                        break
+                            else:
+                                Warning(
+                                    'input argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
+                                        arg))
 
 
 
                     # check for target
                     if len(available_fields)>0:
-                        for i in range(len(self.targets)):
-                            arg = self.targets.key_list[i]
-                            data_feed[arg] = ''
-                            if len(train_data) == 1:
-                                data_feed[self.targets.key_list[0]] = train_data.key_list[0]
-                            elif arg in available_fields:
-                                data_feed[arg] = arg
-                                available_fields.remove(arg)
-                            elif arg == 'target' and 'label' in available_fields:
-                                data_feed[arg] = 'label'
-                                available_fields.remove('label')
-                            elif arg == 'target' and len(available_fields) == 1:
-                                data_feed[arg] = available_fields[0]
-                                available_fields.remove(available_fields[0])
-                            elif len(available_fields) > 0:
-                                target_shape = outshapes[i]
-                                for item in available_fields:
-                                    data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
-                                    if target_shape == data_shape:
-                                        data_feed[arg] = item
-                                        available_fields.remove(item)
-                                    elif ('int64' in str(train_data[item].dtype) or 'int32' in str(
-                                            train_data[item].dtype)) and target_shape[:-1] == data_shape:
-                                        data_feed[arg] = item
-                                        available_fields.remove(item)
-                                    else:
-                                        Warning(
-                                            'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
-                                                arg))
-                        if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
-                            self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
+                        if len(available_fields) == 1:
+                            data_feed['target'] = available_fields[0]
+                        else:
+                            for i in range(len(self.targets)):
+                                arg = self.targets.key_list[i]
+                                data_feed[arg] = ''
+                                if len(train_data) == 1:
+                                    data_feed[self.targets.key_list[0]] = train_data.key_list[0]
+                                elif arg in available_fields:
+                                    data_feed[arg] = arg
+                                    available_fields.remove(arg)
+                                elif arg == 'target' and 'label' in available_fields:
+                                    data_feed[arg] = 'label'
+                                    available_fields.remove('label')
+                                elif arg == 'target' and len(available_fields) == 1:
+                                    data_feed[arg] = available_fields[0]
+                                    available_fields.remove(available_fields[0])
+                                elif len(available_fields) > 0:
+                                    target_shape = outshapes[i]
+                                    for item in available_fields:
+                                        data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
+                                        if target_shape == data_shape:
+                                            data_feed[arg] = item
+                                            available_fields.remove(item)
+                                        elif ('int64' in str(train_data[item].dtype) or 'int32' in str(
+                                                train_data[item].dtype)) and target_shape[:-1] == data_shape:
+                                            data_feed[arg] = item
+                                            available_fields.remove(item)
+                                        else:
+                                            Warning(
+                                                'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
+                                                    arg))
+                            if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
+                                self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
 
                     # if len(self._signature.inputs.key_list) == 1 and data_feed[self._signature.inputs.key_list[0]] != None:
                     #     self.training_context['data_feed'] = data_feed
@@ -695,7 +677,9 @@ class Model(ModelBase):
             for item in train_data.key_list:
                 if item in input_list:
                     # only model 's input argments
-                    train_data[item] = to_tensor(train_data[item].copy(), requires_grad=True)#.cpu()
+                    train_data[item] = to_tensor(train_data[item].copy())#.cpu()
+                    if 'float' in str(train_data[item].dtype):
+                        train_data[item].require_grads=True
                 elif item in self.targets.key_list or data_feed:
                     train_data[item] = to_tensor(train_data[item].copy())#.cpu()
                 else:
@@ -714,22 +698,26 @@ class Model(ModelBase):
         return train_data, test_data
 
     def do_preparation_for_loss(self):
-        # self._model.zero_grad()
-        self.optimizer.zero_grad()
+        pass
+
 
     def get_current_loss(self):
         return self.training_context['current_loss']
 
     def do_gradient_update(self, log_gradients=False):
         try:
-            if self.training_context['stop_update'] <1:
+            if isinstance(self._model,(Layer,nn.Module)):
+                #double check!!!
+                self._model.train()
 
+            if self.training_context['stop_update'] <1:
                 if get_session_value('amp_available') == True and get_session_value('is_amp_enable') == True :
                     if self.gradscaler is None:
                         self.gradscaler=torch.cuda.amp.GradScaler()
                     self.gradscaler.scale(self.training_context['current_loss']).backward(retain_graph=self.training_context['retain_graph'])
                 else:
                     self.training_context['current_loss'].backward(retain_graph=self.training_context['retain_graph'])
+
                 #only check once every epoch start.
                 for callback in self.training_context['callbacks']:
                     callback.on_optimization_step_start(self.training_context)
@@ -756,7 +744,7 @@ class Model(ModelBase):
                     #         PrintException()
                 elif isinstance(self._model,torch.Tensor):
                     grad_norm = self._model.grad.norm()
-                    if not 0 < grad_norm < 1e5:
+                    if not is_tensor(self._model) and not 0 < grad_norm < 1e5:
                         sys.stderr.write('warning...Gradient norm {0} exceed 1e5 nor less-or-equal zero\n'.format(grad_norm))
                         if any_abnormal_number(grad_norm):
                             raise ValueError('grad_norm cannot has abnormal number (nan or inf).')
@@ -822,7 +810,7 @@ class Model(ModelBase):
         if isinstance(self._model, Layer) and any_abnormal_number(self._model):
             for para in self._model.parameters():
                 if any_abnormal_number(para):
-                    where(is_nan(para).all(), random_normal_like(para, mean=0, std=0.02).to(get_device()), para)
+                    para.data.copy_(where(is_nan(para), random_normal_like(para, mean=0, std=0.02).to(get_device()), para))
 
             sys.stderr.write(self._get_name() + '  nan detected!!\n')
 
@@ -878,6 +866,8 @@ class Model(ModelBase):
         else:
             raise ValueError('only Layer or nn.Module as model can export to onnx, yours model is {0}'.format(type(self._model)))
 
+        for callback in self.training_context['callbacks']:
+            callback.on_model_saving_end(self.training_context)
 
     def save_onnx(self, save_path, dynamic_axes=None):
         if isinstance(self._model,nn.Module):
@@ -897,8 +887,9 @@ class Model(ModelBase):
 
             self._model.to(get_device())
             outputs = self._model(dummy_input)
-
-
+            if dynamic_axes is None:
+                dynamic_axes = {self.inputs.key_list[0]: {0: 'batch_size'},  # variable lenght axes
+                                self.outputs.key_list[0]: {0: 'batch_size'}}
 
             # if dynamic_axes is None:
             #     dynamic_axes = {}
@@ -914,31 +905,44 @@ class Model(ModelBase):
                               do_constant_folding=True,  # whether to execute constant folding for optimization
                               input_names=self.inputs.key_list,  # the model's input names
                               output_names=self.outputs.key_list,  # the model's output names
-                              dynamic_axes= {self.inputs.key_list[0] : {0 : 'batch_size'},    # variable lenght axes
-                                self.outputs.key_list[0]: {0 : 'batch_size'}})
+                              dynamic_axes= dynamic_axes)
             self._model.train()
             shutil.copy(save_path, save_path.replace('.onnx_', '.onnx'))
             os.remove(save_path)
+            for callback in self.training_context['callbacks']:
+                callback.on_model_saving_end(self.training_context)
         else:
             raise ValueError('only Layer or nn.Module as model can export to onnx, yours model is {0}'.format(type(self._model)))
 
+
     def load_model(self, file_path):
         print('Loading pretrained model from {}'.format(file_path))
-        pretrained_dict = torch.load(file_path,  map_location=torch.device(get_device()))
-
-        if "state_dict" in pretrained_dict.keys():
-            pretrained_dict = pretrained_dict['state_dict']
+        state_dict = torch.load(file_path,  map_location=torch.device(get_device()))
+        pretrained_dict=None
+        optimizer_dict=None
+        if "state_dict" in state_dict.keys():
+            pretrained_dict = state_dict['state_dict']
+        # if "optimizer_state_dict" in state_dict.keys():
+        #     optimizer_dict = state_dict['state_dict']
+        #
+        # if check_keys(self.optimizer, optimizer_dict):
+        #     self.optimizer.load_state_dict(optimizer_dict, strict=False)
 
         if check_keys(self._model, pretrained_dict):
             has_abnormal=False
             for value in pretrained_dict.values():
                 if is_tensor(value) and any_abnormal_number(value):
                     has_abnormal=True
-                    where(is_nan(value).all(), random_normal_like(value, mean=0, std=0.02).to(get_device()).cast(value.dtype), value)
+                    value=where(is_nan(value), random_normal_like(value, mean=0, std=0.02).to(get_device()).cast(value.dtype), value)
             if has_abnormal:
                 sys.stderr.write(self._model._name+ '  has_abnormal detected and  fixed!!\n')
             self._model.load_state_dict(pretrained_dict, strict=False)
             print('Model loaded!')
+            #must switch to evluate first beforeinference or training
+            #Dropout and Batch normalization will behavior change!!!
+
+            self._model.eval()
+
 
         if self.signature is None:
             self._model.signature = get_signature(self._model.forward)
