@@ -266,7 +266,7 @@ class TrainingPlan(object):
                     PrintException()
                     raise ValueError('save_path:{0} is not valid path'.format(folder))
         plot = PlotLossMetricsCallback(frequency if unit == 'epoch' else -1, frequency if unit == 'batch' else -1,
-                                       save_path=save_path, name_prefix=name_prefix,
+                                       save_path=save_path, name_prefix=name_prefix.format(get_time_suffix()),
                                        clean_ipython_output_frequency=clean_ipython_output_frequency, imshow=imshow)
         self.callbacks.append(plot)
         return self
@@ -307,19 +307,16 @@ class TrainingPlan(object):
                                                     -1]  # -1 is for handel autoencoder scenario
 
                                 trainingitem.training_context['data_feed'] = data_feed
-                                print('data_feed for {0} :{1}'.format(trainingitem.name,
-                                                                      data_feed))  # elif len(outputs)==1
-                    elif len(trainingitem.signature.inputs)  == len(data_loader.signature.outputs) :
+                                print('data_feed for {0} :{1}'.format(trainingitem.name,data_feed))  # elif len(outputs)==1
+                    elif len(trainingitem.signature.inputs)  <= len(data_loader.signature.outputs) :
                         for inp in trainingitem.signature.inputs.key_list:
                             if inp in data_loader.signature.outputs.key_list:
                                 data_feed[inp]=inp
                         if len(data_feed.key_list)==0:
                             for i in range(len(trainingitem.signature.inputs)):
                                 data_feed[trainingitem.signature.inputs.key_list[i]] = data_loader.signature.outputs.key_list[i]
-                    else:
-                        raise RuntimeError(
-                            'the number of models input plus the numbers of  targets should equal to the numbers of '
-                            'dataset items')
+                        trainingitem.training_context['data_feed'] = data_feed
+                        print('data_feed for {0} :{1}'.format(trainingitem.name, data_feed))
                 else:
                     raise RuntimeError('Multiple data loader data_feed auto-generation is not support Now.')
 
@@ -327,7 +324,8 @@ class TrainingPlan(object):
                   keep_weights_history=False, keep_gradient_history=False):
         try:
             self.execution_id = get_time_suffix()
-
+            exception_cnt=0
+            abnormal_num_count=0
             # update callback
             if not is_resume or only_steps == True:
                 for item in self.training_items.values():
@@ -425,7 +423,7 @@ class TrainingPlan(object):
                             if callback.is_shared == True:
                                 callback.on_overall_batch_end(self.__dict__)
 
-                        if self.save_model_frequency > 0 and self.save_model_unit == 'batch' and mbs % \
+                        if self.save_model_frequency > 0 and self.save_model_unit == 'batch' and (num_batches+1) % \
                                 self.save_model_frequency == 0:
                             for k, trainitem in self.training_items.items():
                                 trainitem.save_model(trainitem.training_context['save_path'])
